/*
 *			    Squirt header
 *			    -------------
 *
 *		Jean II '00
 *
 * Public header for the whole library.
 * Defines all you need to know about the "library" ;-)
 * You may want to read some examples to get a better idea on how to
 * use this stuff...
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#ifndef SQUIRT_H
#define SQUIRT_H

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/************************ CONSTANTS & MACROS ************************/

/* ---------------------- PROCESSING OPTIONS ---------------------- */
/* Special keywork */
#define OPT_UNDEFINED		0	/* Let the library use the default */
/* Type of XML generated */
#define OPT_XML_URL		1	/* No XML, plain URL */
#define OPT_XML_OLD		2	/* Old : backward compatibility */
#define OPT_XML_STD		4	/* Standard : most common format */
#define OPT_XML_XLINK		8	/* XLink tentative format */
#define OPT_XML_VCARD		16	/* VCard compatibility */
#define OPT_DO_XML		1	/* Generate/parse received XML */
#define OPT_NO_XML		2	/* Don't generate/parse received XML */
/* Type of transaction generated */
#define OPT_BEACON		1	/* It's a beacon == no action */
#define OPT_SQUIRT		2	/* It's an e-squirt, trigger action */
/* Type of transport and protocol used */
#define OPT_IRDA		1	/* Use the full IrDA stack */
#define OPT_ULTRA		2	/* IrDA-Ultra : connectionless */
#define OPT_BLUETOOTH		4	/* RfComm over BlueTooth */
#define OPT_RAW			1	/* Don't use Obex */
#define OPT_OBEX		2	/* Use Obex Put */
#define OPT_OBEX_GET		4	/* Use Obex Get */
/* A bunch of Obex options */
#define OPT_OBEX_NO_STDIAS	1	/* Don't lookup std Obex server */
#define OPT_OBEX_DO_STDIAS	2	/* Also lookup std Obex server */
#define OPT_OBEX_NO_NAME	1	/* Don't send dummy filename */
#define OPT_OBEX_HAS_NAME	2	/* Send dummy ascii filename */
#define OPT_OBEX_UNICODE_NAME	4	/* Send dummy unicode filename */
#define OPT_OBEX_TYPE_NONE	0x1	/* Don't set payload type */
#define OPT_OBEX_TYPE_USER	0x2	/* User defined type */
#define OPT_OBEX_TYPE_XML	0x4	/* Set "text/xml" */
#define OPT_OBEX_TYPE_REFS	0x8	/* Set "text/x-ref-simple" */
#define OPT_OBEX_TYPE_REFX	0x10	/* Set "text/x-ref-xml" */
#define OPT_OBEX_TYPE_REFL	0x20	/* Set "text/x-ref-list" */
#define OPT_OBEX_TYPE_VCARD	0x40	/* Set "text/x-vcard" */
#define OPT_OBEX_TYPE_WML	0x80	/* Set "text/wml" */
#define OPT_OBEX_TYPE_HTML	0x100	/* Set "text/html" */
#define OPT_OBEX_TYPE_XHTML	0x200	/* Set "text/xhtml" */
#define OPT_OBEX_TYPE_FORM	0x400	/* Set "x-www-form-encoded" */
#define OPT_OBEX_TYPE_UFORM	0x800	/* Set "x-www-form-urlencode" */
#define OPT_OBEX_TYPE_GETOV	0x1000	/* Set "get/name-overflow" */
#define OPT_OBEX_TYPE_POSTOV	0x2000	/* Set "post/name-overflow" */
#define OPT_OBEX_DO_SESSION	1	/* Obex session */
#define OPT_OBEX_NO_SESSION	2	/* No Obex session, single PUT */
#define OPT_OBEX_TARGET_NONE	1	/* No Obex target */
#define OPT_OBEX_TARGET_USER	2	/* User defined target */
#define OPT_OBEX_TARGET_ASCII	4	/* Put Ascii targets */
#define OPT_OBEX_TARGET_UUGLI	8	/* Put UUGLI targets : Unique
					 * Universal Generic Long IDs */

/* Behavior options */
#define OPT_UOS_DUPS_KEEP	1	/* Don't filter ultra-obex dups */
#define OPT_UOS_DUPS_DEL	2	/* Filter ultra-obex squirt dups */
#define OPT_SAUTH_ONDEM		1
#define OPT_SAUTH_ALWAYS	2
#define OPT_SAUTH_NEVER		4
#define OPT_NO_GIFACE		1	/* Do only a PUT */
#define OPT_DO_GIFACE		2	/* Do Get_Interface after put reply */
#define OPT_NO_LINKADDR		1	/* Send to any device */
#define OPT_DO_LINKADDR		2	/* Try to match the linkaddr */
#define OPT_DO_FILTER		1	/* Filter on Obex hint bit or IAS */
#define OPT_NO_FILTER		2	/* Do not filter on Obex hint bit */
/* For get Interface flags */
#define OPT_OBEX_NO_HTTP	1	/* Don't send Obex HTTP header */
#define OPT_OBEX_HAS_HTTP	2	/* Send Obex HTTP header */
#define OPT_OBEX_NO_ACCEPT	1	/* Don't send HTTP-Accept header */
#define OPT_OBEX_HAS_ACCEPT	2	/* Send Obex HTTP-Accept header */
#define OPT_DO_SQUIRT_GET	1	/* Add a squirt in GET request */
#define OPT_NO_SQUIRT_GET	2	/* Don't add squirt in GET request */
#define OPT_DATA_NO_FREE	1	/* Should not free the data */
#define OPT_DATA_DO_FREE	2	/* Should free the data */
/* For RUI functions flags */
#define OPT_RUI_PREFIX_SQUIRT	1	/* String is using "squirt:" format */
#define OPT_RUI_PREFIX_OBEX	2	/* String is using "obex:" format */
#define OPT_RUI_PREFIX_UOBEX	4	/* String is using "uobex:" format */
#define OPT_RUI_FORM_NONE	1	/* This is not a form */
#define OPT_RUI_FORM_GET	2	/* This is a GET form */
#define OPT_RUI_FORM_POST	4	/* This is a POST form */
#define OPT_RUI_FORM_SQUIRT	8	/* This is an e-Squirt !!! */
#define OPT_RUI_BROWSER_NONE	1	/* Don't use browser specific code */
#define OPT_RUI_BROWSER_EUDORA	2	/* Palm Eudora compatibility */

/* ------------------------ VARIOUS LIMITS ------------------------ */
#define ES_MAX_LABEL	128	/* Label in XML string */
#define ES_MAX_URL	512	/* URL in XML string */
#define ES_MAX_TYPE	128	/* Type in XML tring */
#define ES_MAX_XML	1024	/* XML string in Obex BODY header */
#define ES_MAX_FILENAME	512	/* Filename/URL in Obex NAME header */
#define ES_MAX_LINKADDR	64	/* IrDA linkaddr (really 22 char) */
#define ES_MAX_TARGET	64	/* Target in Obex TARGET header */
#define ES_MAX_HTTP	256	/* HTTP string in Obex HTTP header */
#define ES_MAX_ACCEPT	128	/* Accept in HTTP string */

/* Size of the Get Reply (the interface) is dynamic and unlimited */

/****************************** TYPES ******************************/
/*
 * Various structures and type we use externally...
 */

/* ----------------------- DATA STRUCTURES ----------------------- */
/* All the stuff we pass on the air... */

/*
 * Content of a e-Squirt payload, broken down
 */
typedef struct es_squirt
{
  /* XML stuff */
  char		label[ES_MAX_LABEL];	/* Readable label : "CoolTown" */
  char		url[ES_MAX_URL];	/* URL : "http://www.cooltown.com" */
  char		type[ES_MAX_TYPE];	/* Content mime type : "text/html" */
  /* The raw XML string */
  char *	data;		/* Returned data (malloc'ed by us) */
  int		datalen;	/* Length of data */
  int		cleanup;	/* 2 = free the data, 1 = don't do it */
} es_squirt;

/*
 * Obex options common to both operations
 */
typedef struct es_obex_opt
{
  /* A bunch of Obex options */
  int		ias;		/* 2 = try "OBEX", 1 = don't try "OBEX" */
  int		name;		/* 2 = Put a filename, 1 = not */
  int		type;		/* 16 = Obex header type, 1 = not */
  int		http;		/* 2 = Put HTTP header, 1 = not */
  int		accept;		/* 2 = Put HTTP-Accept header, 1 = not */
  int		session;	/* 2 = No Obex session, 1 = Do Obex session */
  int		target;		/* 8 = UUGLI target, 4 = Ascii target */
} es_obex_opt;

/*
 * Options and flags for a squirt or beacon
 */
typedef struct es_opt
{
  /* XML Options */
  int		xml;		/* XML format (User, URL, OLD, JJB) */
  int		do_xml;		/* 2 = don't generate/parse XML, 1 = do it */
  /* Transaction options */
  int		model;		/* 2 = squirt, 1 = beacon, 0 = undefined */
  /* Transport Options */
  int		transport;	/* 4 = BlueTooth, 2 = ultra, 1 = irda */
  int		protocol;	/* 4 = Obex Get, 2 = Obex, 1 = raw */
  /* A bunch of Obex options */
  es_obex_opt	obex;
  /* Behavious options */
  int		uobex_dups;	/* 2 = remove dups, 1 = keep dups */
  int		send_auth;	/* 3 = never, 2 = always, 1 = on demand */
  int		get_iface;	/* 2 = get interface if possible, 1 = never */
  int		linkaddr;	/* 2 = match the linkaddr, 1 = any device */
  int		filter;		/* 2 = Don't filter on hint bit, 1 = do it */
} es_opt;

/*
 * Content of a Get Interface, broken down
 */
typedef struct gi_obget
{
  /* Returned data */
  char *	data;		/* Returned data (malloc'ed by us) */
  int		datalen;	/* Length of data */
  int		cleanup;	/* 2 = free the data, 1 = don't do it */
} gi_obget;

/*
 * Options and flags for a Get Interface
 */
typedef struct gi_flags
{
  /* Transport Options */
  int		transport;	/* 1 = irda */
  int		protocol;	/* 4 = Obex Get, 2 = Obex, 1 = raw */
  /* A bunch of Obex options */
  es_obex_opt	obex;
  /* Behavious options */
  int		squirt_get;	/* 2 = don't send squirt, 1 = send squirt */
  int		reply_auth;	/* 3 = never, 2 = always, 1 = on demand */
  int		linkaddr;	/* 2 = match the linkaddr, 1 = any device */
  int		filter;		/* 2 = Don't filter on hint bit, 1 = do it */
} gi_flags;

/*
 * Optional user defined transport data for squirt/get interface
 */
typedef struct es_trans
{
  /* Obex data */
  char		filename[ES_MAX_FILENAME];	/* Obex filename */
  char		type[ES_MAX_TYPE];		/* Obex type header */
  char *	http_hdr;			/* Obex HTTP header */
  int		http_cleanup;			/* Free/No-free .http */
  char		accept[ES_MAX_ACCEPT];		/* Obex HTTP-Accept header */
  char		target[ES_MAX_TARGET];		/* Obex target header */
  int		targetlen;			/* For non-ascii targets */
  /* IrDA/BlueTooth data */
  char		linkaddr[ES_MAX_LINKADDR];	/* Dest Linkaddr */
  /* Ultra data */
  unsigned int	seq;		/* Sequence number (Ultra Squirt dups) */
  /* Extra */
  void *	userdata;	/* Whatever the user want to do with it */
} es_trans;

/*
 * Content of an authentication, broken down
 * (May change when implemented for real)
 */
typedef struct es_auth
{
  /* Object identification */
  char		user[128];	/* User name */
  char		passwd[128];	/* Password hashed with MD5 */
} es_auth;

/*
 * Data necessary for RUI processing
 */
typedef struct rui_data
{
  char *	action;		/* The RUI string */
  char *	input;		/* The user input */
  int		inputlen;	/* Length of input */
  char *	encoding;	/* Post form encoding */
} rui_data;

/*
 * Data necessary for Form fields processing
 */
typedef struct rui_field
{
  char *	label;		/* Name of the field */
  char *	content;	/* User input in the field */
} rui_field;

/*
 * Options and flags for a rui functions
 */
typedef struct rui_params
{
  int		browser;	/* Adjust for specific browser input */
  int		prefix;		/* 2 = "obex:", 1 = "squirt:" */
  int		form;		/* 4 = POST, 2 = GET, 1 = not a form */ 
} rui_params;

/* ------------------------ DATA CALLBACKS ------------------------ */
/* Receive data from the library */

/*
 * Callback for receiving an e-Squirt or beacon.
 * The first two parameters are those passed to recv_squirt(),
 * The third and fourth parameters are usually NULL.
 * Return :	-1 = failure
 *		 0 = success, no returned Obex headers
 *		 1 = success, returns valid Obex headers,
 *						don't free(request.data)
 *		 2 = success, returns valid Obex headers, do free(request.data)
 */
typedef int (*squirt_handler) (es_squirt *, es_opt *, es_trans *,
			       gi_obget *, gi_flags *);

/*
 * OBEX GET Client callback in the post processing function
 * Receive the filename and type, must return a buffer or NULL
 * Return :	-1 = failure
 *		 1 = success, don't free(request.data)
 *		 2 = success, do free(request.data)
 */
typedef int (*obget_handler) (gi_obget *, gi_flags *, es_trans *, void *);

/*
 * Authentication Client callback in the post processing function
 * May be call to check or reply to an authentication.
 * (May change when implemented for real)
 */
typedef int (*auth_handler) (es_auth *, es_squirt *, es_opt *, es_trans *,
			     gi_obget *, gi_flags *, void *);

/* ----------------------- SERVER CALLBACKS ----------------------- */
/* Receive events from the receiver main loop */

/* Forward declaration */
typedef struct es_listen es_listen;
typedef struct es_post es_post;
typedef struct es_rx_data es_rx_data;

/*
 * Client callback in the main squirt server loop
 * Note : the handler *must not* free the arguments, as they belong
 * to the function calling squirt_receiver()...
 * If return !0, the main server will exit...
 */
typedef int (*accept_handler) (es_post *, es_rx_data *, void *);
/*
 * Client callback in case of select interuption.
 * If a signal is sent to the process, the normal signal handler will
 * catch it and then, if the process is still alive, this handler will
 * be called...
 * Also, if the timeout value is != 0, this will be called each time
 * the timeout expire.
 * First arg is select return value : 0=timeout, -1=signal
 * If return !0, the main server will exit...
 */
typedef int (*select_handler) (int, void *);

/* ---------------------- SERVER STRUCTURES ---------------------- */
/* All the data used to control the receiver behaviour */

struct es_rx_data
{
  /* Receiver structures */
  es_squirt *	content;		/* Rx payload */
  es_opt *	options;		/* Rx Options */
  es_trans *	extra;			/* Rx transport data */
  /* Handlers */
  squirt_handler	callback;	/* Rx Callback */
  /* Listening options */
  es_opt *	listen_options;		/* Listening Options */
};

/*
 * Post processing data
 */
struct es_post
{
  /* Listener data */
  es_listen *	param1;		/* Opaque pointer -> list of nodes */
  es_listen *	param2;		/* Opaque pointer -> current node */
  /* Processing options */
  int		do_post;	/* Run the post handler */
  int		do_cleanup;	/* Close all sockets and free listeners */
};

/*
 * Stuff we optionally pass to recv_squirt to control the behaviour of
 * the receiver main loop...
 */
typedef struct es_server_data
{
  /* Handlers */
  accept_handler	acceptback;	/* Accept Callback */
  select_handler	timeback;	/* Select Callback (signal/timeout) */
  /* Data */
  void *		cbdata;		/* CallBack data */
  int			timeout;	/* Optional select timeout */
} es_server_data;

/**************************** PROTOTYPES ****************************/
/*
 * All the calls that the user application can make
 */

/* ------------------------- SQUIRT CALLS ------------------------- */
/* Send a squirt or a beacon (generic - blocking call) */
extern int
	send_squirt(es_squirt *	content,	/* Tx Payload */
		    es_opt *	options,	/* How to process */
		    es_trans *	extra,		/* Optional Transport data */
		    gi_obget *	reply,		/* Optional Rx Obex headers */
		    gi_flags *	flags);		/* Optional Rx flags */
/* Receiver server : call handler for each incomming squirt or beacon
 * (call is blocking and may be called with NULL callback) */
extern int
	recv_squirt(es_squirt *		content,	/* Rx payload */
		    es_opt *		options,	/* How to listen */
		    es_trans *		extra,		/* Transport data */
		    squirt_handler	callback,	/* Rx Callback */
		    es_server_data *	control);	/* Control server */

/* --------------------- RECV CONTROL SUPPORT --------------------- */
/* --- Most often, you don't need to use those function --- */
/* Update post processing data after a fork() */
extern int
	post_fork_squirt(es_post *	post_data,
			 es_rx_data *	rx_data,
			 int		fpid);		/* fork return code */
/* Duplicate post processing data for multithread model */
extern int
	pre_thread_squirt(es_post *	old_post_data,
			  es_post *	new_post_data,
			  es_rx_data *	old_rx_data,
			  es_rx_data *	new_rx_data,
			  es_squirt *	new_content,	/* New Payload */
			  es_opt *	new_options,	/* New Options */
			  es_trans *	new_extra);	/* New Extra */
/* Perform post processing of incomming squirt or beacon */
extern int
	post_process_squirt(es_post *		post_data,	/* Opaque */
			    es_rx_data *	rx_data);	/* Rx stuff */

/* -------------------------- UTILITIES -------------------------- */
/* Check what the library is able to do */
extern void
	get_capa(es_opt *	options,	/* Squirt Capabilities */
		 gi_flags *	flags);		/* ObGet Capabilities */
/* Initialise the structures with reasonable values */
extern void
	init_structs(es_squirt *	content,
		     es_opt *		options,
		     es_trans *		extra,
		     gi_obget *		request,
		     gi_flags *		flags,
		     es_server_data *	control);
/* Reclaim memory allocated in structures */
extern void
	free_structs(es_squirt *	content,
		     es_trans *		extra,
		     gi_obget *		request);
/* Print out content of a e-Squirt or Beacon */
extern void
	debug_squirt(es_squirt *	content,	/* Payload */
		     es_opt *		options,	/* Options */
		     es_trans *		extra);		/* Extra */
/* Display content of an Get Interface request/reply */
extern void
	debug_obget(gi_obget *	request,
		    gi_flags *	flags,
		    es_trans *	extra);
/* Print out content of a RUI junk */
extern void
	debug_rui(rui_data *	html,		/* The RUI string */
		  rui_params *	params);	/* Processing options */
/* Print out content of a Form */
extern void
	debug_form(rui_field *	field,		/* One field of the form */
		   rui_params *	params);	/* Processing options */
/* Get the length of an unicode string */
extern int
	unicode_strlen(const char *	s);
/* Copy an unicode string around */
extern int
	unicode_strcpy(char *dest, const char *src, int n);
extern int
	unicode_to_char_strcpy(char *dest, const char *src, int n);

/* -------------------- GET INTERFACE SUPPORT -------------------- */
/* --- Note : those functions are only available if Obex is compiled in --- */
/* Request the interface of other device (blocking) */
extern int
	get_interface(gi_obget *	request,	/* What to get */
		      gi_flags *	flags,		/* How to get it */
		      es_trans *	extra);		/* Transport data */
/* Set the handler to catch incomming get interface calls
 * (non blocking - depend on recv_squirt() with proper options) */
extern int
	set_obget_handler(obget_handler	getback,
			  void *	userdata);
/* Set the handlers used for authentication stuff (non implemented) */
extern int
	set_auth_handler(auth_handler	auth_check,	/* Valid. incomm req */
			 auth_handler	auth_reply,	/* Reply to auth req */
			 void *		userdata);

/* --------------------- REFLECTED UI SUPPORT --------------------- */

/* Escape reserved characters from a squirt XML fragment */
extern char *
	escape_uri(char *	dest,		/* Destination buffer */
		   int		dlen,		/* Buffer size */
		   char *	src);		/* String to process */
/* Parse a HREF HTML fragment containing an embedded e-squirt command */
extern int
	rui_parse_href(rui_data *	html,		/* The RUI string */
		       rui_params *	params,		/* Processing opts */
		       es_squirt *	content,	/* Tx Payload */
		       es_opt *		options,	/* How to process */
		       es_trans *	extra,		/* Transport data */
		       gi_obget *	reply,		/* Rx Obex headers */
		       gi_flags *	flags);		/* Rx flags */
/* Parse a ACTION HTML FORM fragment containing an embedded e-squirt */
extern int
	rui_parse_action(rui_data *	html,		/* The RUI string */
			 rui_params *	params,		/* Processing opts */
			 es_squirt *	content,	/* Tx Payload */
			 es_opt *	options,	/* How to process */
			 es_trans *	extra,		/* Transport data */
			 gi_obget *	reply,		/* Rx Obex headers */
			 gi_flags *	flags);		/* Rx flags */
/* Extract the various stuff comming in a RUI request */
extern int
	rui_extract_obget(es_squirt *	content,	/* Tx Payload */
			  es_opt *	options,	/* How to process */
			  es_trans *	extra,		/* Transport data */
			  gi_obget *	reply,		/* Rx Obex headers */
			  gi_flags *	flags,		/* Rx flags */
			  rui_data *	html,		/* The RUI string */
			  rui_params *	params);	/* Processing opts */
/* Extract an input field out of a form string */
extern char *
	rui_extract_field(char *	input,		/* Input string */
			  rui_field *	field);		/* Field content */

/**************************** VARIABLES ****************************/
/* None - Keep it that way */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	/* SQUIRT_H */
