/*
 *			    Squirt common
 *			    -------------
 *
 *		Jean II '00
 *
 * Handle all the common code for the various e-squirt applications.
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

/***************************** INCLUDES *****************************/

#include "squirt_common.h"

/************************ CONSTANTS & MACROS ************************/

static const char *	version	= "squirt_common v2.6 - 24/5/1\n";

/*
 * The different Obex types defined for e-Squirt
 */
static const char *	obex_type_string[] =
{
  "*undefined*",
  "*none*",
  "*user*",
  "text/xml",
  "text/x-ref-simple",
  "text/x-ref-xml",
  "text/x-ref-list",
  "text/x-vcard",
  "text/wml",
  "text/html",
  "text/xhtml",
  "application/x-www-form-encoded",
  "application/x-www-form-urlencode",
  "get/name-overflow",
  "post/name-overflow",
};
static const int OBEX_TYPE_NUM = (sizeof(obex_type_string)/sizeof(char *));

/* Ascii Obex targets */
const unsigned char BEACON_TARGET[] = "hp:beacon";
const unsigned char SQUIRT_TARGET[] = "hp:esquirt";
/* UUIDs for Obex targets */
const unsigned char UUGLI_BEACON_TARGET[] = {0xb7, 0x7a, 0xdd, 0xa9, 0xf7,
					     0xd2, 0x11, 0xd4, 0x9c, 0xd9,
					     0x00, 0x10, 0x83, 0x00, 0xef,
					     0x7b};
const unsigned char UUGLI_SQUIRT_TARGET[] = {0xb7, 0x7a, 0xdd, 0xa8, 0xf7,
					     0xd2, 0x11, 0xd4, 0x9c, 0xd9,
					     0x00, 0x10, 0x83, 0x00, 0xef,
					     0x7b};

static const char *	xml_formats[] = { "undefined", "url", "old", "std",
					  "xlink", "vcard" };
static const char *	squirt_formats[] = { "undefined", "beacon", "squirt" };
static const char *	transport_formats[] = { "undefined", "IrDA", "Ultra",
						"BlueTooth" };
static const char *	obex_formats[] = { "undefined", "raw", "Obex",
					   "Obex-Get" };
static const char *	obex_ias[] = { "undefined", "[OBEX:ESquirt]",
				       "[OBEX]" };
static const char *	obex_session[] = { "undefined", "Yes", "No" };
static const char *	obex_target[] = { "undefined", "none", "user",
					  "ASCII", "UUGLI" };
static const char *	rui_prefix[] = { "undefined", "squirt:", "obex:",
					 "uobex:" };
static const char *	rui_form[] = { "undefined", "none", "get", "post",
				       "e-squirt" };
//static const char *	rui_browser[] = { "undefined", "none", "eudora" };

/************************* RANDOM UTILITIES *************************/
/*
 * Various utilities that don't fall in a specific category...
 * Those are exported...
 */

/* ---------------------------------------------------------------- */
/*
 * Get the options that the library do support
 */
void
get_capa(es_opt *	options,	/* Squirt Capabilities */
	 gi_flags *	flags)		/* ObGet Capabilities */
{
  /* Set all the options that we have */
  options->xml = OPT_XML_URL | OPT_XML_OLD | OPT_XML_STD | OPT_XML_XLINK \
    | OPT_XML_VCARD;
  options->do_xml = OPT_DO_XML | OPT_NO_XML;
  options->model = OPT_BEACON | OPT_SQUIRT;
#ifdef HAS_ULTRA
  options->transport = OPT_IRDA | OPT_ULTRA;
#else /* HAS_ULTRA */
  options->transport = OPT_IRDA;
#endif /* HAS_ULTRA */
#ifdef HAS_OBEX
  options->protocol = OPT_RAW | OPT_OBEX | OPT_OBEX_GET;
#else /* HAS_OBEX */
  options->protocol = OPT_RAW;	/* Note : we may still do Obex over Ultra */
#endif /* HAS_OBEX */
  /* How can we check that one ? By doing some IAS query ? */
  options->obex.ias = OPT_OBEX_NO_STDIAS | OPT_OBEX_DO_STDIAS;
  options->obex.name = OPT_OBEX_NO_NAME | OPT_OBEX_HAS_NAME |
    OPT_OBEX_UNICODE_NAME;
  options->obex.type = OPT_OBEX_TYPE_NONE | OPT_OBEX_TYPE_USER |
    OPT_OBEX_TYPE_XML | 
    OPT_OBEX_TYPE_REFS | OPT_OBEX_TYPE_REFX | OPT_OBEX_TYPE_REFL |
    OPT_OBEX_TYPE_VCARD | OPT_OBEX_TYPE_FORM;
  options->obex.session = OPT_OBEX_DO_SESSION | OPT_OBEX_NO_SESSION;
  options->obex.target = OPT_OBEX_TARGET_ASCII | OPT_OBEX_TARGET_UUGLI;
  options->uobex_dups = OPT_UOS_DUPS_KEEP | OPT_UOS_DUPS_DEL;
  /* This one is not available */
  options->send_auth = 0;
  options->get_iface = OPT_NO_GIFACE | OPT_DO_GIFACE;
  options->linkaddr = OPT_DO_LINKADDR | OPT_NO_LINKADDR;
  options->filter = OPT_DO_FILTER | OPT_NO_FILTER;

  /* And now, for the flags... */
#ifdef HAS_OBEX
  flags->transport = OPT_IRDA;
  flags->protocol = OPT_OBEX_GET;
#else /* HAS_OBEX */
  flags->transport = OPT_UNDEFINED;
  flags->protocol = OPT_UNDEFINED;
#endif /* HAS_OBEX */
  flags->obex.name = OPT_OBEX_NO_NAME | OPT_OBEX_HAS_NAME |
    OPT_OBEX_UNICODE_NAME;
  flags->obex.type = OPT_OBEX_TYPE_NONE | OPT_OBEX_TYPE_USER;
  flags->obex.accept = OPT_OBEX_NO_ACCEPT | OPT_OBEX_HAS_ACCEPT;
  /* How can we check that one ? By doing some IAS query ? */
  flags->obex.ias = OPT_OBEX_NO_STDIAS | OPT_OBEX_DO_STDIAS;
  flags->obex.session = OPT_OBEX_DO_SESSION | OPT_OBEX_NO_SESSION;
  flags->obex.target = OPT_OBEX_TARGET_NONE | OPT_OBEX_TARGET_USER;
  /* This one is not available */
  flags->reply_auth = 0;
  flags->linkaddr = OPT_DO_LINKADDR | OPT_NO_LINKADDR;
  flags->filter = OPT_DO_FILTER | OPT_NO_FILTER;
}

/* ---------------------------------------------------------------- */
/*
 * Initialise the structures with reasonable values
 */
void
init_structs(es_squirt *	content,
	     es_opt *		options,
	     es_trans *		extra,
	     gi_obget *		request,
	     gi_flags *		flags,
	     es_server_data *	control)
{
  /* The es_squirt structure */
  if(content != NULL)
    memset(content, 0, sizeof(es_squirt));
  /* The es_opt structure */
  if(options != NULL)
    {
      memset(options, 0, sizeof(es_opt));
      options->xml = OPT_XML_STD;
    }
  /* The gi_obget structure */
  if(request != NULL)
    {
      memset(request, 0, sizeof(gi_obget));
      request->datalen = -1;
    }
  /* The gi_opt structure */
  if(flags != NULL)
    memset(flags, 0, sizeof(gi_flags));
  /* The es_trans structure */
  if(extra != NULL)
    {
      memset(extra, 0, sizeof(es_trans));
      extra->targetlen = -1;
    }
  /* The es_server_data structure */
  if(control != NULL)
    memset(control, 0, sizeof(es_server_data));
}

/* ---------------------------------------------------------------- */
/*
 * Reclaim memory allocated in structures
 */
void
free_structs(es_squirt *	content,
	     es_trans *		extra,
	     gi_obget *		request)
{
  /* The es_squirt structure */
  if(content != NULL)
    {
      /* Get rid of potential squirt data passed to us */
      if((content->data != NULL) && (content->cleanup == OPT_DATA_DO_FREE))
	{
	  free(content->data);
	  content->data = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(content->data)\n", __FUNCTION__);
#endif
	}
    }

  /* The es_trans structure */
  if(extra != NULL)
    {
      /* Get rid of potential http header passed to us */
      if((extra->http_hdr != NULL) &&
	 (extra->http_cleanup == OPT_DATA_DO_FREE))
	{
	  free(extra->http_hdr);
	  extra->http_hdr = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(extra->http_hdr)\n", __FUNCTION__);
#endif
	}
    }

  /* The gi_obget structure */
  if(request != NULL)
    {
      /* Get rid of potential form data passed to us */
      if((request->data != NULL) && (request->cleanup == OPT_DATA_DO_FREE))
	{
	  free(request->data);
	  request->data = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(request->data)\n", __FUNCTION__);
#endif
	}
    }
}

/* ---------------------------------------------------------------- */
/*
 * Get the length of an unicode string
 * Not optimised. Crash on NULL pointers. Who care...
 */
int
unicode_strlen(const char *	s)
{
  int	i;

  /* For each bundle */
  for(i = 0; i < ES_MAX_XML; i += 2)
    {
      /* Find a NULL in the lower byte */
      if(s[i+1] == '\0')
	return(i / 2);
    }
  return(-1);
}

/* ---------------------------------------------------------------- */
/*
 * Copy an unicode string around
 */
int
unicode_strcpy(char *dest, const char *src, int n)
{
  /* Check null pointers */
  if(dest == NULL)
    return(-1);

  /* Check length */
  if(n < 0)
    n = unicode_strlen(src);
  if(n < 0)
    return(n);

  /* Copy */
  memcpy(dest, src, n*2);

  /* Terminate */
  dest[n*2] = '\0';
  dest[n*2 + 1] = '\0';

  return(n);
}

/* ---------------------------------------------------------------- */
/*
 * Copy an unicode string to a normal string
 * Please check buffer sizes before calling here...
 * This function doesn't crash and burn on non terminated input
 * if you set the size right ;-)
 */
int
unicode_to_char_strcpy(char *dest, const char *src, int n)
{
  int i;

  /* Check null pointers */
  if(dest == NULL)
    return(-1);

  /* Check length */
  if(n < 0)
    n = unicode_strlen(src);
  if(n < 0)
    return(n);

  /* Copy over */
  for(i = 0; i < n; i++)
    dest[i] = src[2*i + 1];
  dest[n] = '\0';
	
  return(n);
}

/******************* OBEX TYPE HEADER PROCESSING *******************/
/*
 * Do the correct stuff for Obex Type header
 * Squirt and get-interface are processed differently due mostly
 * to historical reasons (predefined squirt types). On the other
 * hand, if predefined types are needed for get-interface,
 * it's easy to add...
 */

/* ---------------------------------------------------------------- */
/*
 * Remap a flag or option to an index
 */
static int
option_index(int		binary)
{
  int i = 0;

  /* Get rid of all the bits */
  while(binary != 0)
    {
      /* loop */
      i++;
      binary >>= 1;
    }
  return(i);
}

/* ---------------------------------------------------------------- */
/*
 * Remap an index to a flag or option
 */
static int
index_option(int		value)
{
  int bits = 1;
  int i;

  /* Check that it is sensible to do so */
  if((value <= 0) || (value >= 32))
    {
      fprintf(stderr, "Invalid value %d\n", value);
      return(0);
    }

  /* Set the bit in the right place */
  for(i = 1; i < value; i++)
    bits <<= 1;

  return(bits);
}

/* ---------------------------------------------------------------- */
/*
 * Return the Obex type string corresponding to the Obex options
 */
const char *
get_obex_type(int		obex_type,
	      char *		usertype)
{
  /* Sanity */
  if((obex_type < 0))
    return(NULL);

  /* If none, go out now */
  if((obex_type == OPT_OBEX_TYPE_NONE) || (obex_type == OPT_UNDEFINED))
    return(NULL);

  /* If user defined, just grab it */
  if(obex_type == OPT_OBEX_TYPE_USER)
    {
      if(usertype != NULL)
	return(usertype);
      else
	return(NULL);
    }

  return((char *) obex_type_string[option_index(obex_type)]);
}

/* ---------------------------------------------------------------- */
/*
 * Return the Obex type string corresponding to the options
 */
const char *
get_es_obex_type(es_opt *	options,
		 char *		usertype)
{
  int	obex_type = options->obex.type;

  /* If undefined, do it ourselves */
  if(obex_type == OPT_UNDEFINED)
    {
      /* Check XML format desired */
      switch(options->xml)
	{
	case OPT_XML_URL:
	  obex_type = OPT_OBEX_TYPE_REFS;
	  break;
	case OPT_XML_VCARD:
	  obex_type = OPT_OBEX_TYPE_VCARD;
	  break;
	default:
	  obex_type = OPT_OBEX_TYPE_REFX;
	  break;
	}
    }

  /* Do it... */
  return(get_obex_type(obex_type, usertype));
}

/* ---------------------------------------------------------------- */
/*
 * Return the Obex type option corresponding to the string
 */
void
match_obex_type(const char *	string,
		int		len,
		es_obex_opt *	obex_options,
		char *		usertype)
{
  int	obex_type = OPT_UNDEFINED;
  int	i;

  /* Check no type */
  if(string == NULL)
    {
      obex_options->type = OPT_OBEX_TYPE_NONE;
      return;
    }

  /* Check if match any string */
  for(i = 0; i < OBEX_TYPE_NUM; i++)
    {
      if(!strcmp(string, obex_type_string[i]))
	{
	  obex_type = index_option(i);
	  /*break;*/
	}
    }

  /* If we can, copy the type for the user */
  if((usertype != NULL) && (strlen(string) < ES_MAX_TYPE))
    {
      /* Copy the string with care */
      if(len < 0)
	strcpy(usertype, string);
      else
	{
	  memcpy(usertype, string, len);
	  usertype[len] = '\0';
	}

      /* In we don't know what the type was, tell the user he can read it */
      if(obex_type == OPT_UNDEFINED)
	obex_type = OPT_OBEX_TYPE_USER;
    }

  obex_options->type = obex_type;
  return;
}

/****************** OBEX TARGET HEADER PROCESSING ******************/
/*
 * Same for target header
 * squirt target is based entirely on the model
 * get-interface targert is purely user defined
 */

/* ---------------------------------------------------------------- */
/*
 * Return the Obex Target string corresponding to the options
 */
const char *
get_es_obex_target(es_opt *	options,
		   int *	plen)
{
  const char *	target;
  int		len;

  /* Check the model */
  switch(options->model)
    {
    case OPT_BEACON:
      if(options->obex.target == OPT_OBEX_TARGET_UUGLI)
	{
	  target = UUGLI_BEACON_TARGET;
	  len = sizeof(UUGLI_BEACON_TARGET);
	}
      else
	{
	  target = BEACON_TARGET;
	  len = strlen(BEACON_TARGET);
	}
      break;
    case OPT_SQUIRT:
      if(options->obex.target == OPT_OBEX_TARGET_UUGLI)
	{
	  target = UUGLI_SQUIRT_TARGET;
	  len = sizeof(UUGLI_SQUIRT_TARGET);
	}
      else
	{
	  target = SQUIRT_TARGET;
	  len = strlen(SQUIRT_TARGET);
	}
      break;
    default:
      target = NULL;
      len = 0;
      break;
    }

  /* Done */
  if(plen != NULL)
    *plen = len;
  return(target);
}

/* ---------------------------------------------------------------- */
/*
 * Set the Model and Obex Target options corresponding to the string
 */
void
match_es_obex_target(const char *	string,
		     int		len,
		     es_opt *		options)
{
  /* Check each of the possible targets in turn */
  options->model = OPT_UNDEFINED;
  options->obex.target = OPT_UNDEFINED;
  /* ASCII BEACON */
  if((len == strlen(BEACON_TARGET)) && (!strcmp(string, BEACON_TARGET)))
    {
      options->model = OPT_BEACON;
      options->obex.target = OPT_OBEX_TARGET_ASCII;
      return;
    }
  /* ASCII SQUIRT */
  if((len == strlen(SQUIRT_TARGET)) && (!strcmp(string, SQUIRT_TARGET)))
    {
      options->model = OPT_SQUIRT;
      options->obex.target = OPT_OBEX_TARGET_ASCII;
      return;
    }
  /* UUGLI (Unique Universal Generic Long IDs) */
  if(len == sizeof(UUGLI_BEACON_TARGET))
    {
      /* UUGLI BEACON */
      if(!memcmp(string, UUGLI_BEACON_TARGET, sizeof(UUGLI_BEACON_TARGET)))
	{
	  options->model = OPT_BEACON;
	  options->obex.target = OPT_OBEX_TARGET_UUGLI;
	}
      /* UUGLI SQUIRT */
      if(!memcmp(string, UUGLI_SQUIRT_TARGET, sizeof(UUGLI_BEACON_TARGET)))
	{
	  options->model = OPT_SQUIRT;
	  options->obex.target = OPT_OBEX_TARGET_UUGLI;
	}
    }
  return;
}

/****************** OBEX TARGET HEADER PROCESSING ******************/
/*
 * Same for HTTP header
 * For now, we only process the accept header
 */

/* ---------------------------------------------------------------- */
/*
 * Create the Obex HTTP Header based on user wishes.
 *
 * We can either use a predefined HTTP header, or create it based
 * on various stuff...
 */
char *
build_gi_obex_http(gi_flags *	flags,		/* How to build */
		   es_trans *	extra)		/* Transport Data */
{
  int		tot_len = 0;
  int		offset = 0;

  /* Check if the user already provided us with a valid HTTP header */
  if(flags->obex.http == OPT_OBEX_HAS_HTTP)
    /* Return that */
    return(extra->http_hdr);

  /* First, check if we need to do anything, and compute total length */

  /* Check if there is an HTTP-Accept header */
  if(flags->obex.accept == OPT_OBEX_HAS_ACCEPT)
    tot_len += 10 + strlen(extra->accept);

  /* If no header, bail out */
  if(tot_len <= 0)
    return(NULL);

  /* Most likely, the buffer was not allocated */
  if(extra->http_hdr == NULL)
    {
      /* This will be big enough */
      extra->http_hdr = malloc(tot_len + 1);
      extra->http_cleanup = OPT_DATA_DO_FREE;
#ifdef DEBUG_ALLOC
      fprintf(stderr, "%s: malloc(%d)\n", __FUNCTION__, tot_len);
#endif
    }

  /* Now build the HTTP header */

  /* Check if there is an HTTP-Accept header */
  if(flags->obex.accept == OPT_OBEX_HAS_ACCEPT)
    {
      sprintf(extra->http_hdr + offset, "Accept: %s\r\n", extra->accept);
      offset += 10 + strlen(extra->accept);
    }

  /* Return the HTTP header that we have built */
  return(extra->http_hdr);
}

/* ---------------------------------------------------------------- */
/*
 * Parse the Obex HTTP Header.
 *
 * This is *not* a compliant and full HTTP header parser. It handles
 * only a single Accept header (optionally on multiple lines), if
 * there is additional HTTP header in there we will fail or put
 * garbage in extra->accept.
 * If you have a compliant HTTP parser, you can plug it there.
 * Alternatively, you can do it in your app by using extra->http...
 *
 * Note : we assume that the HTTP header contains only
 * the Accept header. Doing the right stuff is complex.
 * Note : we match the ':' because HTTP defines some
 * Accept-XXX header like Accept-Charset, that's ok
 * because HTTP require the colon here (without a space before).
 */
void
parse_gi_obex_http(const char *		string,
		   int			len,
		   gi_flags *		flags,
		   es_trans *		extra)
{
  /* Init */
  flags->obex.http   = OPT_OBEX_NO_HTTP;
  flags->obex.accept = OPT_OBEX_NO_ACCEPT;

  /* Check if the user want to get those... */
  if(extra == NULL)
    {
      flags->obex.http   = OPT_UNDEFINED;	/* == error */
      flags->obex.accept = OPT_UNDEFINED;	/* == error */
      return;
    }

  /* Always realloc http buffer */
  if(extra->http_hdr != NULL)
    {
      fprintf(stderr, "%s: BUG !\n", __FUNCTION__);      
    }
  extra->http_hdr = malloc(len + 1);
  extra->http_cleanup = OPT_DATA_DO_FREE;
#ifdef DEBUG_ALLOC
  fprintf(stderr, "%s: malloc(%d)\n", __FUNCTION__, len);
#endif

  /* Get the http header in full */
  memcpy(extra->http_hdr, string, len);
  extra->http_hdr[len] = '\0';
  flags->obex.http = OPT_OBEX_HAS_HTTP;

  /* Minimal check to see if we have the right guy... */
  if((len > 7) && (!strncmp(string, "Accept:", 7)))
    {
      /* Remove the "Accept:" prefix - crude but effective */
      len -= 7;
      string += 7;

      /* Remove leading whitespace */
      while(isspace(string[0]))
	{
	  string++;
	  len--;
	}
      /* Remove trailing whitespace, including \r\n */
      while(isspace(string[len - 1]))
	len--;

      /* Copy the accept header */
      if(len < ES_MAX_ACCEPT)
	{
	  memcpy(extra->accept, string, len);
	  extra->accept[len] = '\0';

	  flags->obex.accept = OPT_OBEX_HAS_ACCEPT;
	}
      else
	{
	  fprintf(stderr, "HTTP-Accept header too big (%d)...\n", len);
	  flags->obex.accept = OPT_UNDEFINED;	/* == error */
	}
    }
}


/************************* DEBUG FACILITIES *************************/

/* ---------------------------------------------------------------- */
/*
 * Print out some hex field...
 */
void
dump_hex(unsigned char *	buf,
	 int			len)
{
  int		i;

  /* Print each character */
  for(i = 0; i < (len - 1); i++)
    {
      fprintf(stderr, "%02X:", buf[i]);
      if((i % 25) == 24)
	printf("\n");
    }
  fprintf(stderr, "%02X\n", buf[i]);
}

/* ---------------------------------------------------------------- */
/*
 * Print out a target...
 */
void
dump_target(unsigned char *	target,
	    int			len)
{
  int		i;

  /* Check the length */
  if(len < 0)
    fprintf(stderr, "%s", target);
  else
    {
      /* Print each character */
      for(i = 0; i < len; i++)
	{
	  fprintf(stderr, "%02X", target[i]);
	}
    }
}

/* ---------------------------------------------------------------- */
/*
 * Print out content of a e-Squirt or Beacon
 */
void
debug_squirt(es_squirt *	content,
	     es_opt *		options,
	     es_trans *		extra)
{
  fprintf(stderr, "Got a e-Squirt or Beacon !!!\n");
  fprintf(stderr, "  -> label     = [%s]\n", content->label);
  fprintf(stderr, "  -> url       = [%s]\n", content->url);
  fprintf(stderr, "  -> type      = [%s]\n", content->type);
  fprintf(stderr, "  -> length    = [%d:%c]\n", content->datalen,
	  content->cleanup == OPT_DATA_DO_FREE ? 'd' : 's');
  fprintf(stderr, "  -> xml       = [%s]\n",
	  content->data ? (content->datalen < 200 ?
			   content->data :
			   "<Too Big>") :
	  "<NULL>");
  if(options->obex.name == OPT_OBEX_HAS_NAME)
    fprintf(stderr, "  -> filename  = [%s] (Ascii)\n", extra->filename);
  if(options->obex.name == OPT_OBEX_UNICODE_NAME)
    {
      char	filename[ES_MAX_FILENAME];
      unicode_to_char_strcpy(filename, extra->filename, -1);
      fprintf(stderr, "  -> filename  = [%s] (Unicode)\n", filename);
    }
  if(options->obex.type != OPT_OBEX_TYPE_NONE)
    {
      if(options->obex.type != OPT_OBEX_TYPE_USER)
	fprintf(stderr, "  -> obex_type = [%s]\n",
		obex_type_string[option_index(options->obex.type)]);
      else
	fprintf(stderr, "  -> obex_type = [%s] (user)\n", extra->type);
    }
  if(extra != NULL)
    fprintf(stderr, "  -> seq       = [0x%X]\n", extra->seq);
  fprintf(stderr, "  with options : xml=%s ; model=%s\n",
	  xml_formats[option_index(options->xml)],
	  squirt_formats[option_index(options->model)]);
  fprintf(stderr, "                 protocol=%s ; transport=%s\n",
	  obex_formats[option_index(options->protocol)],
	  transport_formats[option_index(options->transport)]);
  fprintf(stderr, "                 Obex IAS=%s ; Obex session=%s\n",
	  obex_ias[option_index(options->obex.ias)],
	  obex_session[option_index(options->obex.session)]);
  fprintf(stderr, "                 Obex Target=%s\n",
	  obex_target[option_index(options->obex.target)]);
  fprintf(stderr, "Version : %s\n", version);
}

/* ---------------------------------------------------------------- */
/*
 * Print out content of a e-Squirt or Beacon
 */
void
debug_obget(gi_obget *	request,
	    gi_flags *	flags,
	    es_trans *	extra)
{
  fprintf(stderr, "Got an interface !!!\n");
  if(flags->obex.name == OPT_OBEX_HAS_NAME)
    fprintf(stderr, "  -> filename  = [%s] (Ascii)\n", extra->filename);
  if(flags->obex.name == OPT_OBEX_UNICODE_NAME)
    {
      char	filename[ES_MAX_FILENAME];
      unicode_to_char_strcpy(filename, extra->filename, -1);
      fprintf(stderr, "  -> filename  = [%s] (Unicode)\n", filename);
    }
  /* Targets are pain in the **** */
  if(flags->obex.target == OPT_OBEX_TARGET_USER)
    {
      fprintf(stderr, "  -> target    = [");
      dump_target(extra->target, extra->targetlen);
      fprintf(stderr, "]\n");
    }
  if(flags->obex.type != OPT_OBEX_TYPE_NONE)
    {
      if(flags->obex.type != OPT_OBEX_TYPE_USER)
	fprintf(stderr, "  -> obex_type = [%s]\n",
		obex_type_string[option_index(flags->obex.type)]);
      else
	fprintf(stderr, "  -> obex_type = [%s] (user)\n", extra->type);
    }
  if(flags->obex.http == OPT_OBEX_HAS_HTTP)
    fprintf(stderr, "  -> http      = [%s]\n",
	    extra->http_hdr ? extra->http_hdr : "<NULL>");
  if(flags->obex.accept == OPT_OBEX_HAS_ACCEPT)
    fprintf(stderr, "  -> accept    = [%s]\n", extra->accept);
  fprintf(stderr, "  -> length    = [%d:%c]\n", request->datalen,
	  request->cleanup == OPT_DATA_DO_FREE ? 'd' : 's');
  fprintf(stderr, "  -> interface = [%s]\n",
	  request->data ? (request->datalen < 200 ?
			   request->data :
			   "<Too Big>") :
	  "<NULL>");
  fprintf(stderr, "  with flags : protocol=%s ; transport=%s\n",
	  obex_formats[option_index(flags->protocol)],
	  transport_formats[option_index(flags->transport)]);
  fprintf(stderr, "               squirt_get=%s\n",
	  obex_session[option_index(flags->squirt_get)]);
  fprintf(stderr, "               Obex IAS=%s ; Obex session=%s\n",
	  obex_ias[option_index(flags->obex.ias)],
	  obex_session[option_index(flags->obex.session)]);
  fprintf(stderr, "\n");
}

/* ---------------------------------------------------------------- */
/*
 * Print out content of a RUI junk
 */
void
debug_rui(rui_data *	html,		/* The RUI string */
	  rui_params *	params)		/* Processing options */
{
  fprintf(stderr, "Got an RUI stuff\n");
  if(html->action)
    fprintf(stderr, "  -> action    = [%s]\n", html->action);
  if(html->input)
    fprintf(stderr, "  -> input     = [%s] (%d)\n",
	    html->inputlen < 200 ? html->input : "<Too Big>",
	    html->inputlen);
  if(html->encoding)
    fprintf(stderr, "  -> encoding  = [%s]\n", html->encoding);

  fprintf(stderr, "  with params : prefix=%s ; form=%s\n",
	  rui_prefix[option_index(params->prefix)],
	  rui_form[option_index(params->form)]);
  fprintf(stderr, "\n");
}

/* ---------------------------------------------------------------- */
/*
 * Print out content of a Form
 */
void
debug_form(rui_field *	field,		/* One field of the form */
	   rui_params *	params)		/* Processing options */
{
  fprintf(stderr, "Got an Form Field\n");
  if(field->label)
    fprintf(stderr, "  -> label    = [%s]\n", field->label);
  if(field->content)
    fprintf(stderr, "  -> content  = [%s]\n", field->content);

  fprintf(stderr, "\n");
}
