/*
 *			    Squirt Obex common
 *			    ------------------
 *
 *		Jean II '00
 *
 * Private header for sobex_common.c
 * Use this if you want more "intimate" access with the guts of the library.
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#ifndef SQUIRT_OBEX_P_H
#define SQUIRT_OBEX_P_H

/***************************** INCLUDES *****************************/

#include "squirt_common.h"

#include <openobex/obex.h>

/************************ CONSTANTS & MACROS ************************/

/*
 * This is the Ir service name (in IrIAS) that we are using for
 * e-Squirt on top of Obex.
 * Note : under Linux, we *never* use "OBEX" as this is reserved for the
 * "regular" Obex server and clients.
 */
#define OBEX_SERVICE_NAME	"OBEX:ESquirt"
#define OBEX_STD_SERVICE_NAME	"OBEX"

/* Obex processing status */
#define OBEX_STAT_CONT		0x0	/* Not finished, continue processing */
#define OBEX_STAT_ERROR		0x1	/* Processing/Link error */
#define OBEX_STAT_ACCEPT	0x2	/* Need to do an accept */
#define OBEX_STAT_SENT		0x3	/* Send PUT completed */
#define OBEX_STAT_RECV		0x4	/* Receive PUT completed */
#define OBEX_STAT_GET		0x5	/* GET received */

/* Missing from OpenObex... */
#define OBEX_HDR_HTTP		0x47 /* Any HTTP header */

/****************************** TYPES ******************************/
/*
 * Various structures and type we use internally...
 */

/*
 * Content of a regular Obex PUT, broken down
 */
typedef struct es_obex
{
  char *	payload;	/* XML string, see above */
  int		size;		/* Size of payload */
  char *	filename;	/* Filename of obex object */
  char *	usertype;	/* User defined type, if any */
  int		transport;	/* 4 = BlueTooth, 1 = irda */
} es_obex;

/*
 * Data exchanged with the Obex sender callback
 */
typedef struct es_obexsend
{
  obex_t *		handle;		/* Obex sender instance */
  int			status;		/* State of Obex processing */
  int			put;		/* Put has been done */
  gi_obget *		reply;		/* Returned headers */
  gi_flags *		flags;		/* Returned flags */
  es_trans *		extra;		/* Returned transport data */
  es_opt *		options;	/* Listen Options */
  int			has_replied;	/* There is some returned headers */
} es_obexsend;

/*
 * Data exchanged with the Obex receiver callback
 */
typedef struct es_obexrecv
{
  /* Processing data */
  obex_t *		handle;		/* Obex sender instance */
  int			status;		/* State of Obex processing */
  int			put;		/* Put has been done */
  int			get;		/* Get has been done */
  int			session;	/* 2 = No Obex session, 1 = Yes */
  /* Received data */
  struct es_obex	data;		/* Received payload */
  es_opt *		options;	/* Receive Options */
  es_rx_data *		userdata;	/* Stuff pertaining to the user */
} es_obexrecv;

/*
 * Data exchanged with the Obex GET sender callback
 */
typedef struct gi_obgetsend
{
  obex_t *		handle;		/* Obex sender instance */
  int			status;		/* State of Obex processing */
  int			get;		/* Successfull reply */
  gi_obget *		request;	/* Request description */
  gi_flags *		flags;		/* Request flags */
  es_trans *		extra;		/* Request transport data */
} gi_obgetsend;

/**************************** PROTOTYPES ****************************/

/* FIXME */
/* --------------------------- OBEX PUT --------------------------- */
extern int
	create_obex_headers(obex_t *		handle,
			    obex_object_t *	object,
			    es_obex *		content,
			    es_opt *		options);
extern void
	obex_client_event(obex_t *		handle,
			  obex_object_t *	object,
			  int			mode,
			  int			event,
			  int			obex_cmd,
			  int			obex_rsp);
extern int
	obex_send_object(es_obexsend *		cbdata,
			 obex_object_t *	object);
extern void
	obex_server_event(obex_t *		handle,
			  obex_object_t *	object,
			  int			mode,
			  int			event,
			  int			obex_cmd,
			  int			obex_rsp);
extern int
	parse_obex_object(obex_t *		handle,
			  obex_object_t *	object,
			  es_obex *		data,
			  es_opt *		options,
			  es_opt *		listen_options);
extern void
	obex_recv_event(obex_t *		handle,
			obex_object_t *		object,
			int			mode,
			int			event,
			int			obex_cmd,
			int			obex_rsp);
/* --------------------------- OBEX GET --------------------------- */
extern int
	create_obget_headers(obex_t *		handle,
			     obex_object_t *	object,
			     gi_obget *		request,
			     gi_flags *		options,
			     es_trans *		extra);
extern void
	obget_client_event(obex_t *		handle,
			   obex_object_t *	object,
			   int			mode,
			   int			event,
			   int			obex_cmd,
			   int			obex_rsp);
extern int
	obget_send_object(gi_obgetsend *	cbdata,
			  obex_object_t *	object);
extern int
	simple_get_interface(obex_t *		handle,
			     gi_obget *		request,
			     gi_flags *		flags,
			     es_trans *		extra);
extern int
	parse_obget_header(obex_t *		handle,
			   obex_object_t *	object,
			   gi_obget *		request,
			   gi_flags *		flags,
			   es_trans *		extra,
			   es_opt *		listen_options);
extern int
	check_get_request(obex_t *		handle,
			  obex_object_t *	object);
extern int
	process_get_request(obex_t *		handle,
			    obex_object_t *	object,
			    es_obex *		put,
			    es_opt *		options,
			    es_rx_data *	userdata);

/**************************** VARIABLES ****************************/

/* Client function we need to call in case of an Obex Get */
extern obget_handler	get_interface_callback;

/* Client function we need to call to check authentication of
 * incomming requests */
extern auth_handler	auth_check_callback;
/* Client function we need to call to reply to an authentication request */
extern auth_handler	auth_reply_callback;

#endif	/* SQUIRT_OBEX_P_H */
