/*
 *			    Squirt Obex Get common
 *			    ----------------------
 *
 *		Jean II '01
 *
 * Handle all the common Obex Get code for the various e-squirt applications.
 *
 * This file is released under the GPL license.
 * This code is derived from the Obex PUT code in squirt_obex_put.c
 * and use functions from there...
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include "squirt_obex.p.h"

/**************************** VARIABLES ****************************/
/* See squirt_obex.p.h */
obget_handler		get_interface_callback = NULL;
void *			get_interface_userdata = NULL;
auth_handler		auth_check_callback = NULL;
auth_handler		auth_reply_callback = NULL;
void *			auth_userdata = NULL;

/********************* OBEX SEND GET INTERFACE *********************/
/*
 * Here we deal with the client part of Get-Interface
 */

/* ---------------------------------------------------------------- */
/*
 * Set the headers of a GET request
 */
int
create_obget_headers(obex_t *		handle,
		     obex_object_t *	object,
		     gi_obget *		request,
		     gi_flags *		flags,
		     es_trans *		extra)
{
  obex_headerdata_t	hdd;		/* Header data */
  char			ucname[ES_MAX_FILENAME * 2];	/* Name in unicode */
  int			hdr_len;

  /* Check if there is a target */
  if(flags->obex.target == OPT_OBEX_TARGET_USER)
    {
      /* Get length of target */
      hdr_len = extra->targetlen;
      if(hdr_len < 0)
	hdr_len = strlen(extra->target);	/* Don't send the '\0' */

      /* Add to header */
      hdd.bs = extra->target;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TARGET, hdd, hdr_len, 0);
    }

  /* Check if there is a name - ASCII */
  if(flags->obex.name == OPT_OBEX_HAS_NAME)
    {
      /* Convert filename in unicode */
      hdr_len = OBEX_CharToUnicode(ucname, extra->filename,
				   sizeof(ucname));

      /* Add it to the header */
      hdd.bs = ucname;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME, hdd,
			   hdr_len, 0);
    }
  /* Check if there is a name - Unicode */
  if(flags->obex.name == OPT_OBEX_UNICODE_NAME)
    {
      hdd.bs = extra->filename;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME,
			   hdd, unicode_strlen(hdd.bs)*2 + 2, 0);
    }

  /* Set the type - in theory, applies only if there is a body,
   * but let's trust that the user do the right thing.
   * Ok, maybe I'm too naive out there... */
  hdd.bs = get_obex_type(flags->obex.type, extra->type);
  if(hdd.bs != NULL)
    {
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TYPE,
			   hdd, strlen(hdd.bs) + 1, 0);
    }

  /* Check if there is an HTTP header */
  hdd.bs = build_gi_obex_http(flags, extra);
  if(hdd.bs != NULL)
    {
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_HTTP, hdd,
			   strlen(hdd.bs), 0);

      /* Note : we need to do the cleanup here, because
       * we need to receive http headers later */
      if(extra->http_cleanup == OPT_DATA_DO_FREE)
	{
	  free(extra->http_hdr);
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(extra->http_hdr)\n", __FUNCTION__);
#endif
	}
      extra->http_hdr = NULL;
    }

  /* If there is some data in the request, just set it in the body */
  if((request->datalen >= 0) && (request->data != NULL))
    {
      /* Put the length */
      hdd.bq4 = request->datalen;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_LENGTH,
			   hdd, sizeof(uint32_t), 0);

      /* And also the content */
      hdd.bs = request->data;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY,
			   hdd, request->datalen, 0);

      /* Note : we need to do the cleanup here, because
       * we need to receive data in request struct in caller */
      if(request->cleanup == OPT_DATA_DO_FREE)
	{
	  free(request->data);
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(request->data)\n", __FUNCTION__);
#endif
	}
      request->data = NULL;
    }

  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Get the payload out of an Obex reply and put it in the request struct...
 */
int
extract_obget_object(obex_t *		handle,
		     obex_object_t *	object,
		     gi_obget *		request,
		     gi_flags *		flags,
		     es_trans *		extra)
{
  obex_headerdata_t	hv;
  uint8_t		hi;
  int			hlen;
  int			done = 0;
  int			ret;
  int			old_obex_name = flags->obex.name;

  /* Init */
  flags->obex.target = OPT_UNDEFINED;
  flags->obex.name = OPT_OBEX_NO_NAME;
  flags->obex.type = OPT_OBEX_TYPE_NONE;
  flags->obex.accept = OPT_UNDEFINED;
  flags->obex.http = OPT_UNDEFINED;

  /* Process all headers */
  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))
    {
      switch(hi)
	{
	case OBEX_HDR_BODY:
	  /* Check length */
	  if(hlen >= 0)
	    {
	      /* Allocate a buffer for the file */
	      request->data = (char *) malloc(hlen + 1);
	      if(request->data == NULL)
		{
		  fprintf(stderr, "Malloc failed\n");
		  return(0);
		}
#ifdef DEBUG_ALLOC
	      fprintf(stderr, "%s : malloc(%d)\n", __FUNCTION__, hlen + 1);
#endif

	      /* Copy over */
	      memcpy(request->data, hv.bs, hlen);
	      /* Terminate the string */
	      request->data[hlen] = '\0';

	      /* Set the length */
	      request->datalen = hlen; 
	      request->cleanup = OPT_DATA_DO_FREE;	/* User do the free */

	      done = 1;
	    }
	  break;

	case OBEX_HDR_LENGTH:
	  /* We know about it, and we don't care... */
	  break;

	case OBEX_HDR_NAME:
	  ret = -2;
	  /* Check what we do with regard to Unicode */
	  if(old_obex_name == OPT_OBEX_UNICODE_NAME)
	    {
	      /* No Unicode translation */
	      if((extra != NULL) && (hlen < (ES_MAX_FILENAME - 1)))
		ret = unicode_strcpy(extra->filename, hv.bs, hlen/2);
	      flags->obex.name = OPT_OBEX_UNICODE_NAME;
	    }
	  else
	    {
	      /* Translate Unicode */
	      if((extra != NULL) && ((hlen/2) < ES_MAX_FILENAME))
		ret = unicode_to_char_strcpy(extra->filename,
					       hv.bs, hlen/2);
	      flags->obex.name = OPT_OBEX_HAS_NAME;
	    }
	  /* Error */
	  if(ret < 0)
	    {
	      fprintf(stderr, "Name too big (%d)...\n", hlen/2);
	      flags->obex.name = OPT_UNDEFINED;	/* == error */
	    }
	  else
	    done = 1;
	  break;

	case OBEX_HDR_TYPE:
	  match_gi_obex_type(hv.bs, hlen, flags, extra->type);
	  break;

	case OBEX_HDR_HTTP:
	  /* Extract what we need from the header */
	  parse_gi_obex_http(hv.bs, hlen, flags, extra);
	  break;

	default:
	  fprintf(stderr, "Skipped header %02x\n", hi);
	}
    }

  return(done);
}

/* ---------------------------------------------------------------- */
/*
 * Even CallBack from the Obex library
 * Only used on the Get-Interface sender side...
 */
void
obget_client_event(obex_t *		handle,
		   obex_object_t *	object,
		   int			mode,
		   int			event,
		   int			obex_cmd,
		   int			obex_rsp)
{
  gi_obgetsend *	cbdata;

  cbdata = OBEX_GetUserData(handle);
  if(cbdata == NULL)
    return;
  cbdata->status = OBEX_STAT_ERROR;	/* Default : error... */

  switch(event)
    {
    case OBEX_EV_PROGRESS:
      /* This one is usually never called */
      cbdata->status = OBEX_STAT_CONT;
      break;
    case OBEX_EV_REQDONE:
      /* Comes when a command has finished. */
      if(obex_rsp == OBEX_RSP_SUCCESS)
	{
	  cbdata->status = OBEX_STAT_SENT;
	  if(obex_cmd == OBEX_CMD_GET)
	    {
	      cbdata->get = extract_obget_object(handle, object,
						 cbdata->request,
						 cbdata->flags,
						 cbdata->extra);
	    }
	}
      else
	{
	  fprintf(stderr, "Obex command not accepted\n");
	  if(obex_cmd == OBEX_CMD_GET)
	    cbdata->get = -(obex_rsp);
	}
      break;
    case OBEX_EV_LINKERR:
      fprintf(stderr, "Link broken\n");
      break;
    default:
      fprintf(stderr, "Unknown event 0x%X\n", event);
      break;
    }
}

/* ---------------------------------------------------------------- */
/*
 * Perform the Obex request and process the answer...
 */
int
obget_send_object(gi_obgetsend *	cbdata,
		  obex_object_t *	object)
{
  int ret;

  /* Reset our flag */
  cbdata->status = OBEX_STAT_CONT;

  /* Do the request */
  OBEX_Request(cbdata->handle, object);

  /* Get answer from Obex lib */
  while(cbdata->status == OBEX_STAT_CONT)
    {
      ret = OBEX_HandleInput(cbdata->handle, 1);
      if(ret < 0)
	return(ret);
    }
  return(cbdata->status);
}

/* ---------------------------------------------------------------- */
/*
 * Issue a get-interface command and return the result, if possible...
 * This one do it within an existing session, so the caller must
 * connect and initialise things properly...
 */
int
simple_get_interface(obex_t *		handle,		/* Obex session */
		     gi_obget *		request,	/* What to get */
		     gi_flags *		flags,		/* How to get it */
		     es_trans *		extra)		/* Transport data */
{
  obex_object_t *	object;		/* Obex object to send */
  gi_obgetsend		cbdata;		/* Data for our callback... */
  int			ret = 0;

  /* Set as user data of the Obex handle */
  cbdata.handle = handle;
  cbdata.status = OBEX_STAT_CONT;
  cbdata.get = 0;
  cbdata.request = request;
  cbdata.flags = flags;
  cbdata.extra = extra;
  /* Set handler and data to do a get-interface */
  OBEX_SetUserCallBack(handle, obget_client_event, &cbdata);

  /* Build Obex GET object containing the request parameters */
  object = OBEX_ObjectNew(handle, OBEX_CMD_GET);
  create_obget_headers(handle, object, request, flags, extra);

  /* Process the "get" (send it and get answer back) */
  ret = obget_send_object(&cbdata, object);

  /* The end - The data will be in the request struct, if any */
  if(cbdata.get == 0)
    return(-2);
  return(0);
}


/********************* OBEX RECV GET INTERFACE *********************/
/*
 * Here, we deal with all the get interface stuff.
 * We depend on the Obex put receiver code to call us...
 */

/* ---------------------------------------------------------------- */
/*
 * Look in a GET request and extract the various part we are
 * interested in...
 */
int
parse_obget_header(obex_t *		handle,
		   obex_object_t *	object,
		   gi_obget *		request,
		   gi_flags *		flags,
		   es_trans *		extra,
		   es_opt *		listen_options)
{
  obex_headerdata_t	hv;
  uint8_t		hi;
  int			hlen;
  int			hasbody = 0;	/* No-body out there... */
  int			ret;

  /* Init */
  request->data = NULL;
  request->datalen = -1;
  flags->obex.target = OPT_OBEX_TARGET_NONE;
  flags->obex.name = OPT_OBEX_NO_NAME;
  flags->obex.type = OPT_OBEX_TYPE_NONE;
  flags->obex.accept = OPT_UNDEFINED;
  flags->obex.http = OPT_UNDEFINED;

  /* Process all headers */
  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))
    {
      switch(hi)
	{
	case OBEX_HDR_TARGET:
	  if((extra != NULL) && (hlen < ES_MAX_TARGET))
	    {
	      memcpy(extra->target, hv.bs, hlen);
	      extra->target[hlen] = '\0';
	      extra->targetlen = hlen;
	      flags->obex.target = OPT_OBEX_TARGET_USER;
	    }
	  else
	    {
	      fprintf(stderr, "Target too big (%d)...\n", hlen);
	      flags->obex.target = OPT_UNDEFINED;	/* == error */
	    }
	  break;

	case OBEX_HDR_TYPE:
	  /* Check if it is one of the various types we know about.
	   * The caller code depend on us doing a good job here... */
	  match_gi_obex_type(hv.bs, hlen, flags, extra->type);
	  break;

	case OBEX_HDR_NAME:
	  ret = -2;
	  /* Check what we do with regard to Unicode */
	  if(listen_options->obex.name == OPT_OBEX_UNICODE_NAME)
	    {
	      /* No Unicode translation */
	      if((extra != NULL) && (hlen < (ES_MAX_FILENAME - 1)))
		ret = unicode_strcpy(extra->filename, hv.bs, hlen/2);
	      flags->obex.name = OPT_OBEX_UNICODE_NAME;
	    }
	  else
	    {
	      /* Translate Unicode */
	      if((extra != NULL) && ((hlen/2) < ES_MAX_FILENAME))
		ret = unicode_to_char_strcpy(extra->filename,
					     hv.bs, hlen/2);
	      flags->obex.name = OPT_OBEX_HAS_NAME;
	    }
	  /* Error */
	  if(ret < 0)
	    {
	      fprintf(stderr, "Name too big (%d)...\n", hlen/2);
	      flags->obex.name = OPT_UNDEFINED;	/* == error */
	    }
	  break;

	case OBEX_HDR_HTTP:
	  /* Extract what we need from the header */
	  parse_gi_obex_http(hv.bs, hlen, flags, extra);
	  break;

	case OBEX_HDR_BODY:
	  hasbody = 1;
	  break;

	case OBEX_HDR_LENGTH:
	  /* We know about it, and we don't care... */
	  break;

	default:
	  fprintf(stderr, "Skipped header %02x\n", hi);
	}
    }

  return(hasbody);
}
	
/* ---------------------------------------------------------------- */
/*
 * If the GET request was not a reference and did contain a body,
 * extract it in request struct.
 * This is either a name overflow (no type) or a POST form (typed)
 */
int
parse_obget_body(obex_t *		handle,
		 obex_object_t *	object,
		 gi_obget *		request)
{
  obex_headerdata_t	hv;
  uint8_t		hi;
  int			hlen;
  int			ret = -1;

  request->data = NULL;
  request->datalen = -1;

  /* Process all headers - again */
  while(OBEX_ObjectGetNextHeader(handle, object, &hi, &hv, &hlen))
    {
      switch(hi)
	{
	case OBEX_HDR_BODY:
	  /* Check length */
	  if(hlen >= 0)
	    {
	      /* Allocate a buffer for the file */
	      request->data = (char *) malloc(hlen + 1);
	      if(request->data == NULL)
		{
		  fprintf(stderr, "Malloc failed\n");
		  return(-1);
		}
#ifdef DEBUG_ALLOC
	      fprintf(stderr, "%s : malloc(%d)\n", __FUNCTION__, hlen + 1);
#endif

	      /* Copy over */
	      memcpy(request->data, hv.bs, hlen);
	      /* Terminate the string */
	      request->data[hlen] = '\0';

	      /* Set the length and cleanup */
	      request->datalen = hlen; 
	      request->cleanup = OPT_DATA_DO_FREE;	/* User do the free */

	      ret = 1;
	    }
	  break;

	case OBEX_HDR_TARGET:
	case OBEX_HDR_NAME:
	case OBEX_HDR_HTTP:
	case OBEX_HDR_TYPE:
	case OBEX_HDR_LENGTH:
	default:
	  /* Already done - ignore */
	  break;
	}
    }

  return(ret);
}
	
/* ---------------------------------------------------------------- */
/*
 * Set the headers and body of a GET reply
 */
int
create_obget_reply(obex_t *		handle,
		   obex_object_t *	object,
		   gi_obget *		request,
		   gi_flags *		flags,
		   es_trans *		extra)
{
  obex_headerdata_t	hv;		/* Header data */
  char			ucname[ES_MAX_FILENAME * 2];	/* Name in unicode */
  int			hdr_len;

  /* Check if there is a name - ASCII */
  if(flags->obex.name == OPT_OBEX_HAS_NAME)
    {
      /* Convert filename in unicode */
      hdr_len = OBEX_CharToUnicode(ucname, extra->filename,
				   sizeof(ucname));

      /* Add it to the header */
      hv.bs = ucname;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME,
			   hv, hdr_len, 0);
    }
  /* Check if there is a name - Unicode */
  if(flags->obex.name == OPT_OBEX_UNICODE_NAME)
    {
      hv.bs = extra->filename;
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_NAME,
			   hv, unicode_strlen(hv.bs)*2 + 2, 0);
    }

  /* Check if there is a type */
  hv.bs = get_obex_type(flags->obex.type, extra->type);
  if(hv.bs != NULL)
    {
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_TYPE,
			   hv, strlen(hv.bs) + 1, 0);
    }

  /* Check if there is an HTTP header */
  hv.bs = build_gi_obex_http(flags, extra);
  if(hv.bs != NULL)
    {
      OBEX_ObjectAddHeader(handle, object, OBEX_HDR_HTTP, hv,
			   strlen(hv.bs), 0);

      /* Cleanup before returning to the server */
      if(extra->http_cleanup == OPT_DATA_DO_FREE)
	{
	  free(extra->http_hdr);
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(extra->http_hdr)\n", __FUNCTION__);
#endif
	}
      extra->http_hdr = NULL;
    }

  /* Put the length */
  hv.bq4 = request->datalen;
  OBEX_ObjectAddHeader(handle, object, OBEX_HDR_LENGTH,
		       hv, sizeof(uint32_t), 0);

  /* And finally the content */
  hv.bs = request->data;
  OBEX_ObjectAddHeader(handle, object, OBEX_HDR_BODY, hv, request->datalen, 0);

  /* Cleanup before returning to the server */
  if(request->cleanup == OPT_DATA_DO_FREE)
    {
      free(request->data);
#ifdef DEBUG_ALLOC
      fprintf(stderr, "%s: free(request->data)\n", __FUNCTION__);
#endif
    }
  request->data = NULL;

  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Verify if we can handle an incomming GET request...
 */
int
check_get_request(obex_t *		handle,
		  obex_object_t *	object)
{
  if(get_interface_callback != NULL)
    return(1);
  else
    return(0);
}

/* ---------------------------------------------------------------- */
/*
 * When we receive an incomming GET, obex_recv_event() will call this
 * function. We decapsulate the request and pass it to user space.
 * Note on the return value (and value of validput) :
 *	-1 == error
 *	 0 == ok, callback not called (so the user has not got the data yet)
 *	 1 == ok, callback called (the user has already a copy of the data)
 */
int
process_get_request(obex_t *		handle,
		    obex_object_t *	object,
		    es_obex *		put,
		    es_opt *		options,
		    es_rx_data *	userdata)
{
  gi_obget		request;
  gi_flags		flags;
  es_trans *		extra = userdata->extra;
  int			hasbody;
  int			validput = 0;	/* No put data received */
  int			ret = -1;

  /* Quick init */
  memset(&request, 0, sizeof(request));
  memset(&flags, 0, sizeof(flags));
  /* Set our flags */
  flags.protocol = OPT_OBEX_GET;
  flags.transport = put->transport;
  flags.obex.session = options->obex.session;
  flags.obex.ias = options->obex.ias;

  /* Get the attributes of the request */
  hasbody = parse_obget_header(handle, object, &request, &flags, extra,
			       userdata->listen_options);

  /* If we found a body (dead or alive), try to extract it */
  if(hasbody)
    {
      /* Reset the object headers */
      OBEX_ObjectReParseHeaders(handle, object);

      /* This is messy. We may have multiple kind of stuff in the body.
       * -> It may be a reference that was pushed.
       * -> It may be a file name or URL to get.
       * -> It may be the user input of a POST form.
       * How to know which is which ? The type field...
       */
      switch(flags.obex.type)
	{
	case OPT_OBEX_TYPE_REFS:
	case OPT_OBEX_TYPE_REFX:
	case OPT_OBEX_TYPE_REFL:
	  /* This is a reference pushed to us. */
	  /* Default is unknow, let the user space deal with that... */

	  /* Decapsulate e-Squirt data */
	  validput = parse_obex_object(handle, object, put, options,
				       userdata->listen_options);
	  break;

	case OPT_OBEX_TYPE_NONE:
	default:
	  /* No type identifies NAME overflows
	   * Any other type is a POST form */

	  /* Decapsulate body input in request struct */
	  ret = parse_obget_body(handle, object, &request);
	  break;

	}
    }

  /* If this is a squirt, let's try to process it ! */
  ret = 0;
  if(validput)
    {
      /* Hook the receive buffer */
      if(userdata->content->data == NULL)
	{
	  userdata->content->data = put->payload;
	  userdata->content->cleanup = OPT_DATA_DO_FREE;
	}
      userdata->content->datalen = put->size;

      /* Correct options */
      options->protocol = OPT_OBEX_GET;
      /* Blank out some conflicting fields */
      options->obex.name = OPT_OBEX_NO_NAME;

      /* Call the user callback if possible */
      if(userdata->callback != (squirt_handler) NULL)
	{
	  /* Process XML and Call user */
	  ret = call_rcv_callback(userdata, &request, &flags);
	  /* Check if callback was successful... */
	  if(ret >= 0)
	    validput = 2;	/* Callback called */
	  else
	    validput = ret;	/* Failure */
	  /* At this point, we have an interface if ret > 0
	   * The user may also set only the filename in the above callback,
	   * because we will call get_interface_callback() anyway... */

	  /* In parse_obex_object(), we may allocate a buffer (data->payload),
	   * which may be freed in call_rcv_callback(), or the callback
	   * may change it. So, get back the modified values. */
	  put->payload = userdata->content->data;
	  put->size = userdata->content->datalen;
	}
    }

  /* Do we have something to send back ??? */
  if((ret > 0) && (request.datalen >= 0) && (request.data != NULL))
    /* Yes, we got everything we need */
    ;
  else
    /* No : Call the user to get an interface (get-interface callback) */
    if(get_interface_callback != NULL)
      ret = get_interface_callback(&request, &flags, extra,
				   get_interface_userdata);

  /* Check what the user returned... */
  if((ret > 0) && (request.datalen >= 0) && (request.data != NULL))
    {
      /* Send the data back (+cleanup userdata) */
      create_obget_reply(handle, object, &request, &flags, extra);

      /* Success !!! */
      OBEX_ObjectSetRsp(object, OBEX_RSP_CONTINUE, OBEX_RSP_SUCCESS);
    }
  else
    {
      /* Failure code : it's a bit tricky. This is the way it works :
       *	o NOT_FOUND -> Everything failed
       *	o NO_CONTENT -> squirt ok, no control page
       *	o SUCCESS -> squirt ok (if any), get back a control page
       * This way, the sender knows at which stage we produced an error,
       * Where it's get messy is that the callback may override the value...
       */
      /* By default, send back "page not found" */
      int	obex_err = OBEX_RSP_NOT_FOUND;
      /* If the squirt was accepted, we are returning "no content". */
      if(validput == 2)
	obex_err = OBEX_RSP_NO_CONTENT;
      /* And if the callback did specify a return code, use that instead */
      if(ret < -10)
	obex_err = -ret;
      OBEX_ObjectSetRsp(object, obex_err, obex_err);
    }

  return(validput - 1);
}

/* ---------------------------------------------------------------- */
/*
 * Set the get interface handler
 * Here, we only set the callback. This callback will only be used
 * if the user start an Obex listener (server), and then for each
 * incomming GET request...
 */
int
set_obget_handler_obex(obget_handler	getback,
		       void *		userdata)
{
  /* Just do it */
  get_interface_callback = getback;
  get_interface_userdata = userdata;

  return(0);
}


/*********************** OBEX AUTHENTICATION ***********************/
/*
 * Here, we mostly set the get interface and authentication handler,
 * most of the hard job will be dealt with in the Obex part...
 */

/* ---------------------------------------------------------------- */
/*
 * Set the authentication handlers
 */
int
set_auth_handler(auth_handler	auth_check,	/* Validate incomming stuff */
		 auth_handler	auth_reply,	/* Reply to an auth query */
		 void *		userdata)
{
  /* FIXME */
  return(-1);
}
