/*
 *			    Push a file over Obex
 *			    ---------------------
 *
 *		Jean II '99 & '00 & '03
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <squirt.h>

/************************* FILE SUBROUTINES *************************/
/*
 * Extracted from obex_io.c, (c) Pontus Fuchs <pontus.fuchs@tactel.se>
 */

/* ---------------------------------------------------------------- */
/*
 * Get the filesize in a "portable" way
 */
static inline int
get_filesize(const char *	filename)
{
  struct stat stats;

  /*  Need to know the file length */
  stat(filename, &stats);

  return (int) stats.st_size;
}

/* ---------------------------------------------------------------- */
/*
 * Read a file and alloc a buffer for it
 */
static char *
easy_readfile(const char *	filename,
	      int *		file_size)
{
  int		actual;
  int		fd;
  char *	buf;

  *file_size = get_filesize(filename);
  printf("name=%s, size=%d\n", filename, *file_size);

  fd = open(filename, O_RDONLY, 0);
  if(fd == -1)
    return(NULL);

  buf = malloc(*file_size);
  if(buf == NULL)
    return(NULL);

  actual = read(fd, buf, *file_size);
  close(fd); 

  *file_size = actual;

  return(buf);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	ret_hdr;
  gi_flags	ret_flags;
  int		ret = -1;

  init_structs(&fields, &options, &extra, &ret_hdr, &ret_flags, NULL);
  /* Set the options */
  options.do_xml = OPT_NO_XML;	/* Don't handle XML in the lib */
  options.transport = OPT_BLUETOOTH | OPT_IRDA;
  options.protocol = OPT_OBEX;
  options.obex.type = OPT_OBEX_TYPE_NONE;
  options.obex.name = OPT_OBEX_HAS_NAME;
  options.obex.session = OPT_OBEX_DO_SESSION;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    default:
      /* Fallthrough */
    case 1:
      /* No argument : error */
      printf("Error : %s filename type linkaddr\n", argv[0]);
      return(-1);

    case 4:
      /* Third argument : Link Address */
      strcpy(extra.linkaddr, argv[3]);
      options.linkaddr = OPT_DO_LINKADDR;
      options.transport = OPT_BLUETOOTH;

      /* Fallthrough */
    case 3:
      /* Second argument : Mime Type */
      strcpy(extra.type, argv[2]);
      options.obex.type = OPT_OBEX_TYPE_USER;

      /* Fallthrough */
    case 2:
      /* One argument : the filename */
      strcpy(extra.filename, argv[1]);
      options.obex.name = OPT_OBEX_HAS_NAME;

      break;
    }

  /* Allocate and read the file */
  fields.data = easy_readfile(extra.filename, &fields.datalen);
  fields.cleanup = OPT_DATA_DO_FREE;
  if(fields.data == NULL)
    {
      printf("Reading file failed...\n");
      return(-1);
    }

  /* Send the file */
  ret = send_squirt(&fields, &options, &extra, &ret_hdr, &ret_flags);
  if(ret < 0)
    printf("Sending file failed...\n");
  else
    if(ret > 0)
      {
	/* Check what we have */
	debug_obget(&ret_hdr, &ret_flags, &extra);
	/* Cleanup */
	free_structs(NULL, &extra, &ret_hdr);
      }

  return(ret);
}
