/*
 *			    IrDA Obex Squirt
 *			    ----------------
 *
 *		Jean II '99 & '00
 *
 * Send and receive e-Squirt using Obex.
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <squirt.h>

/*#define BROWSER		"lynx %s"*/
//#define BROWSER		"netscape %s &"

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
server_accept(es_post *		post_data,	/* Opaque */
	      es_rx_data *	rx_data,	/* Content, option and co */
	      void *		cbdata)		/* CallBack data */
{
  int		ret = -1;

  printf("Accept done...\n");

  /* Perform post processing */
  ret = post_process_squirt(post_data, rx_data);

  /* Check... */
  debug_squirt(rx_data->content, rx_data->options, rx_data->extra);

  /* Return back to the server, 0 -> continue */
  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
receive_esquirt(es_squirt *	fields,		/* Rx payload */
		es_opt *	options,	/* Rx options */
		es_trans *	extra,		/* Rx transport data */
		gi_obget *	ret_hdr,	/* Not used */
		gi_flags *	ret_flags)	/* Not used */
{
  debug_squirt(fields, options, extra);

#ifdef BROWSER
  {
    unsigned char		buf[512];	/* The command */
    sprintf(buf, BROWSER, fields->url);
    system(buf);
  }
#endif

  /* Return back to the server */
  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a Get Interface request.
 */
int
reply_obget(gi_obget *	request,
	    gi_flags *	flags,
	    es_trans *	extra,
	    void *	userdata)
{
  static char	string[65536];
  int		n = 65535;

  /* Check what we have */
  debug_obget(request, flags, extra);

  /* Get rid of potential form data passed to us */
  free_structs(NULL, extra, request);

  /* Set the name */
  strcpy(extra->filename, "index.html");
  flags->obex.name = OPT_OBEX_HAS_NAME;
  /* Set the type */
  strcpy(extra->type, "text/html");
  flags->obex.type = OPT_OBEX_TYPE_USER;

  /* Set the data */
  memset(string, 'T', n);
  string[n] = '\0';
  request->data = string;
  /* Set the length (not including '\0') */
  request->datalen = n;
  request->cleanup = OPT_DATA_NO_FREE;

  /* Return back to the server */
  return(1);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  es_server_data	control;
  int		n = 0;
  int		ret = -1;

  init_structs(&fields, &options, &extra, NULL, NULL, NULL);
  /* Set the options */
  options.do_xml = OPT_NO_XML;	/* Don't handle XML in the lib */
  options.model = OPT_UNDEFINED;	/* Only for the receiver */
  options.transport = OPT_IRDA;
  options.protocol = OPT_OBEX;

  memset(&control, 0, sizeof(control));
  control.acceptback = &server_accept;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    case 1:
      /* No argument : receive a beacon */
      break;
    default:
      /* Fallthrough */
    case 2:
      /* One argument : number of char to send */
      sscanf(argv[1], "%d", &n);
      fields.datalen = n + 1;
      fields.data = malloc(n + 1);
      memset(fields.data, 'J', n);
      fields.data[n] = '\0';
      fields.cleanup = OPT_DATA_DO_FREE;
      break;
    }

  if(argc > 1)
    {
      /* Send a single e-Squirt/Beacon */
      options.model = OPT_SQUIRT;	/* Make sure model is defined */
      ret = send_squirt(&fields, &options, &extra, NULL, NULL);
      if(ret < 0)
	printf("Sending e-Squirt failed...\n");
    }
  else
    {
      /* Prepare to handle get interface calls */
      set_obget_handler(&reply_obget, NULL);

      /* Receive multiple e-Squirt/Beacon */
      printf("Waiting for e-Squirts over Obex...\n");
      recv_squirt(&fields, &options, &extra, &receive_esquirt, &control);
      /* Will loop forever */
    }

  return(ret);
}
