/*
 *			    Generic Squirt
 *			    --------------
 *
 *		Jean II '99 & '00
 *
 * Send and receive e-Squirt using all options...
 * Now, the generic code has been move to squirt_common.c...
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <squirt.h>

/*#define BROWSER		"lynx %s"*/
//#define BROWSER		"netscape %s &"

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
server_accept(es_post *		post_data,	/* Opaque */
	      es_rx_data *	rx_data,	/* Content, option and co */
	      void *		cbdata)		/* CallBack data */
{
  int		ret = -1;

  printf("Accept done...\n");

  /* Perform post processing */
  ret = post_process_squirt(post_data, rx_data);

  /* Return back to the server, 0 -> continue */
  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon.
 */
int
receive_esquirt(es_squirt *	fields,		/* Rx payload */
		es_opt *	options,	/* Rx options */
		es_trans *	extra,		/* Rx transport data */
		gi_obget *	ret_hdr,	/* Return filename */
		gi_flags *	ret_flags)	/* Return flags */
{
  int		ret = 0;	/* Reply Header not set */

  /* Display content */
  debug_squirt(fields, options, extra);

#ifdef BROWSER
  {
    unsigned char		buf[512];	/* The command */
    sprintf(buf, BROWSER, fields->url);
    system(buf);
  }
#endif

  /* Check if we can return some data
   * We could also check if it is OPT_OBEX or OPT_OBEX_GET */
  if(ret_hdr != (gi_obget *) NULL)
    {
      /* Set the name */
      strcpy(extra->filename, "index.html");
      ret_flags->obex.name = OPT_OBEX_HAS_NAME;
      /* Set the types we support (only applicable for OPT_OBEX) */
      strcpy(extra->accept, "text/html, text/xml");
      ret_flags->obex.accept = OPT_OBEX_HAS_ACCEPT;

      ret = 1;	/* Reply Header set */
    }

  /* Return back to the server */
  return(ret);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a Get Interface request.
 */
int
reply_obget(gi_obget *	request,
	    gi_flags *	flags,
	    es_trans *	extra,
	    void *	userdata)
{
  static char	string[] = "<HTML><HEAD><TITLE>My Interface</TITLE></HEAD>\n<BODY>No much yet ;-)</BODY></HTML>";

  /* Check what we have */
  debug_obget(request, flags, extra);

  /* Note : 
   * If it is a simple get-interface, the requester may have set name and
   * type in its request, and we should take this into account.
   * If it is a squirt-get, receive_esquirt() will have already set
   * name and type in the structure */

  /* Get rid of potential form data passed to us */
  free_structs(NULL, extra, request);

  /* Set the name */
  strcpy(extra->filename, "index.html");
  flags->obex.name = OPT_OBEX_HAS_NAME;
  /* Set the type */
  strcpy(extra->type, "text/html");
  flags->obex.type = OPT_OBEX_TYPE_USER;

  /* Set the data */
  request->data = string;
  /* Set the length (not including '\0') */
  request->datalen = strlen(string);
  request->cleanup = OPT_DATA_NO_FREE;

  /* Return back to the server */
  return(1);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	ret_hdr;
  gi_flags	ret_flags;
  es_server_data	control;
  int		ret = -1;

  init_structs(&fields, &options, &extra, &ret_hdr, &ret_flags, &control);
  /* Set the options */
  options.xml = OPT_XML_STD;
  options.model = OPT_UNDEFINED;	/* Only for the receiver */
  options.transport = OPT_UNDEFINED;
  options.protocol = OPT_UNDEFINED;
  options.obex.type = OPT_UNDEFINED;	/* auto */
  options.get_iface = OPT_DO_GIFACE;

  control.acceptback = &server_accept;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    case 1:
      /* No argument : receive a beacon */
      break;
    case 2:
      /* One argument : the URL */
      strcpy(fields.url, argv[1]);
      break;
    case 3:
      /* Two arguments : label + url */
      strcpy(fields.label, argv[1]);
      strcpy(fields.url, argv[2]);
      break;
    default:
      /* Fallthrough */
    case 4:
      /* Three arguments : label + url + type */
      strcpy(fields.label, argv[1]);
      strcpy(fields.url, argv[2]);
      strcpy(fields.type, argv[3]);
      break;
    }

  if(argc > 1)
    {
      /* Send a single e-Squirt/Beacon */
      options.model = OPT_SQUIRT;	/* Make sure model is defined */
      ret = send_squirt(&fields, &options, &extra, &ret_hdr, &ret_flags);
      if(ret < 0)
	printf("Sending e-Squirt failed...\n");
      else
	if(ret > 0)
	  {
	    /* Check what we have */
	    debug_obget(&ret_hdr, &ret_flags, &extra);
	    /* Cleanup */
	    free_structs(NULL, &extra, &ret_hdr);
	  }
    }
  else
    {
      /* Prepare to handle get interface calls */
      set_obget_handler(&reply_obget, NULL);

      /* Receive multiple e-Squirt/Beacon */
      printf("Waiting for e-Squirts over all transports...\n");
      recv_squirt(&fields, &options, &extra, &receive_esquirt, &control);
      /* Will loop forever */
    }

  return(ret);
}
