/*
 *			    IrDA Obex Squirt a FORM string
 *			    ------------------------------
 *
 *		Jean II '99, '00 & '01
 *
 * Parse a HREF string and send it Obex GET.
 * 
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <squirt.h>

/*#define BROWSER		"lynx %s"*/
//#define BROWSER		"netscape %s &"

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
server_accept(es_post *		post_data,	/* Opaque */
	      es_rx_data *	rx_data,	/* Content, option and co */
	      void *		cbdata)		/* CallBack data */
{
  int		ret = -1;

  printf("Accept done...\n");

  /* Perform post processing */
  ret = post_process_squirt(post_data, rx_data);

  /* Return back to the server, 0 -> continue */
  return(0);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
receive_esquirt(es_squirt *	fields,		/* Rx payload */
		es_opt *	options,	/* Rx options */
		es_trans *	extra,		/* Rx transport data */
		gi_obget *	ret_hdr,	/* Not used */
		gi_flags *	ret_flags)	/* Not used */
{
  int		ret = 0;	/* Reply Header not set */

  debug_squirt(fields, options, extra);

#ifdef BROWSER
  {
    unsigned char		buf[512];	/* The command */
    sprintf(buf, BROWSER, fields->url);
    system(buf);
  }
#endif

  /* Check if we can return some data */
  if(ret_hdr != (gi_obget *) NULL)
    {
      /* Set the name */
      strcpy(extra->filename, "index.html");
      ret_flags->obex.name = OPT_OBEX_HAS_NAME;
      /* Set the types we support (only applicable for OPT_OBEX) */
      strcpy(extra->accept, "text/html, text/xml");
      ret_flags->obex.accept = OPT_OBEX_HAS_ACCEPT;

      ret = 1;	/* Reply Header set */
    }

  /* Return back to the server */
  return(ret);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a Get Interface request.
 */
int
reply_obget(gi_obget *	request,
	    gi_flags *	flags,
	    es_trans *	extra,
	    void *	userdata)
{
  static char	string[] = "<HTML><HEAD><TITLE>My Interface</TITLE></HEAD>\n<BODY>No much yet ;-)</BODY></HTML>";
  rui_data	html;		/* The RUI string */
  rui_params	params;		/* Processing opts */
  es_squirt	fields;
  es_opt	options;

  /* Check what we have */
  debug_obget(request, flags, extra);

  /* Note : 
   * If it is a simple get-interface, the requester may have set name and
   * type in its request, and we should take this into account.
   * If it is a squirt-get, receive_esquirt() will have already set
   * name and type in the structure */

  /* Ok, now decapsulate the form data... */
  init_structs(&fields, &options, NULL, NULL, NULL, NULL);
  rui_extract_obget(&fields, &options, extra, request, flags,
		    &html, &params);
  debug_rui(&html, &params);
  if(params.form == OPT_RUI_FORM_SQUIRT)
    /* Here we process the reference or we call receive_squirt() */
    debug_squirt(&fields, &options, extra);
  if((params.form == OPT_RUI_FORM_GET) ||
     (params.form == OPT_RUI_FORM_POST))
    {
      char *		next = html.input;
      rui_field		ffield;
      /* Parse the form data */
      do
	{
	  next = rui_extract_field(next, &ffield);
	  debug_form(&ffield, NULL);
	}
      while(next != NULL);
    }

  /* Get rid of potential form data passed to us (only after processed it) */
  free_structs(NULL, extra, request);

  /* Set the name */
  strcpy(extra->filename, "index.html");
  flags->obex.name = OPT_OBEX_HAS_NAME;
  /* Set the type */
  flags->obex.type = OPT_OBEX_TYPE_HTML;

  /* Set the data */
  request->data = string;
  /* Set the length (not including '\0') */
  request->datalen = strlen(string);
  request->cleanup = OPT_DATA_NO_FREE;

  /* Return back to the server */
  return(1);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	interface;
  gi_flags	flags;
  rui_data	html;		/* The RUI string */
  rui_params	params;		/* Processing opts */
  es_server_data	control;
  int		ret = -1;

  init_structs(&fields, &options, &extra, &interface, &flags, &control);

  control.acceptback = &server_accept;

  /* Look if there is any command line args */
  if(argc > 1)
    {
      fprintf(stderr, " ---- Parsing command line... ----\n\n");

      /* First arg is the type of form processing */
      ret = -1;
      if((!strcmp(argv[1], "Eudora")) && (argc >= 3))
	{
	  params.browser = OPT_RUI_BROWSER_EUDORA;
	  params.form    = OPT_RUI_FORM_NONE;	/* Will get overwritten */
	  ret = 0;
	}
      if((!strcasecmp(argv[1], "None")) && (argc >= 3))
	{
	  params.browser = OPT_RUI_BROWSER_NONE;
	  params.form    = OPT_RUI_FORM_NONE;
	  ret = 0;
	}
       if((!strcasecmp(argv[1], "Get")) && (argc >= 4))
	{
	  params.browser = OPT_RUI_BROWSER_NONE;
	  params.form    = OPT_RUI_FORM_GET;
	  ret = 0;
	}
      if((!strcasecmp(argv[1], "Post")) && (argc >= 4))
	{
	  params.browser = OPT_RUI_BROWSER_NONE;
	  params.form    = OPT_RUI_FORM_POST;
	  ret = 0;
	}
      if(ret < 0)
	{
	  fprintf(stderr, "Not a known form\n");
	  return(-1);
	}

      /* Parse what we have */
      params.prefix = OPT_RUI_PREFIX_OBEX;
      html.action = argv[2];
      html.input = argv[3];
      if(params.form != OPT_RUI_FORM_NONE)
	html.inputlen = strlen(html.input);
      html.encoding = argv[4];
      if(rui_parse_action(&html, &params,
			  &fields, &options, &extra, &interface, &flags) < 0)
	return(-1);

      /* A few extra options... */
      options.obex.ias = OPT_OBEX_DO_STDIAS;
      options.obex.target = OPT_OBEX_TARGET_UUGLI;
      flags.obex.ias = OPT_OBEX_DO_STDIAS;

      /* Display what has been done */
      debug_squirt(&fields, &options, &extra);
      debug_obget(&interface, &flags, &extra);
      fprintf(stderr, " ---- Now, trying to send this... ----\n\n");

      /* Send a single e-Squirt/Beacon and get associated interface */
      ret = send_squirt(&fields, &options, &extra, &interface, &flags);

      /* Display it */
      if(ret < 0)
	printf("Sending e-Squirt failed...\n");
      else
	{
	  /* Check what we have */
	  debug_obget(&interface, &flags, &extra);
	  /* Cleanup */
	  free_structs(NULL, &extra, &interface);
	}
    }
  else
    {
      /* Set the receive options that matter */
      options.model = OPT_UNDEFINED;	/* Only for the receiver */
      options.transport = OPT_UNDEFINED;
      options.protocol = OPT_UNDEFINED;

      /* Prepare to handle get interface calls */
      set_obget_handler(&reply_obget, NULL);

      /* Receive multiple e-Squirt/Beacon */
      printf("Waiting for e-Squirts over Obex...\n");
      recv_squirt(&fields, &options, &extra, &receive_esquirt, &control);
      /* Will loop forever */
    }

  return(ret);
}
