/*
 *			    IrDA Obex Squirt+Get
 *			    --------------------
 *
 *		Jean II '99 & '00
 *
 * Send and receive e-Squirt using Obex GET.
 * This is ugly and prone to confusion, but it's in the spec...
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include <squirt.h>

/*#define BROWSER		"lynx %s"*/
//#define BROWSER		"netscape %s &"

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
receive_esquirt(es_squirt *	fields,		/* Rx payload */
		es_opt *	options,	/* Rx options */
		es_trans *	extra,		/* Rx transport data */
		gi_obget *	ret_hdr,	/* Not used */
		gi_flags *	ret_flags)	/* Not used */
{
  int		ret = 0;	/* Reply Header not set */

  debug_squirt(fields, options, extra);

#ifdef BROWSER
  {
    unsigned char		buf[512];	/* The command */
    sprintf(buf, BROWSER, fields->url);
    system(buf);
  }
#endif

  /* Check if we can return some data */
  if(ret_hdr != (gi_obget *) NULL)
    {
      /* Set the name */
      strcpy(extra->filename, "index.html");
      ret_flags->obex.name = OPT_OBEX_HAS_NAME;
      /* Set the types we support (only applicable for OPT_OBEX) */
      strcpy(extra->accept, "text/html, text/xml");
      ret_flags->obex.accept = OPT_OBEX_HAS_ACCEPT;

      ret = 1;	/* Reply Header set */
    }

  /* Return back to the server */
  return(ret);
}

/* ---------------------------------------------------------------- */
/*
 * Receive a Get Interface request.
 */
int
reply_obget(gi_obget *	request,
	    gi_flags *	flags,
	    es_trans *	extra,
	    void *	userdata)
{
  static char	string[65536];
  int		n = 65535;

  /* Check what we have */
  debug_obget(request, flags, extra);

  /* Get rid of potential form data passed to us */
  free_structs(NULL, extra, request);

  /* Set the name */
  strcpy(extra->filename, "index.html");
  flags->obex.name = OPT_OBEX_HAS_NAME;
  /* Set the type */
  strcpy(extra->type, "text/html");
  flags->obex.type = OPT_OBEX_TYPE_USER;

  /* Set the data */
  memset(string, 'T', n);
  string[n] = '\0';
  request->data = string;
  /* Set the length (not including '\0') */
  request->datalen = n;
  request->cleanup = OPT_DATA_NO_FREE;

  /* Return back to the server */
  return(1);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	interface;
  gi_flags	flags;
  int		n = 0;
  int		ret = -1;

  init_structs(&fields, &options, &extra, &interface, &flags, NULL);
  /* Set the options */
  options.do_xml = OPT_NO_XML;	/* Don't handle XML in the lib */
  options.model = OPT_UNDEFINED;	/* Only for the receiver */
  options.transport = OPT_IRDA;
  options.protocol = OPT_OBEX_GET;
  options.obex.ias = OPT_OBEX_DO_STDIAS;
  options.obex.name = OPT_OBEX_NO_NAME;
  options.obex.type = OPT_UNDEFINED;

  /* Set the fields */
  strcpy(extra.filename, "test.html");
  strcpy(extra.accept, "text/html, text/xml");
  strcpy(extra.target, "hp:test");
  interface.data = NULL;
  interface.datalen = -1;
  flags.obex.name = OPT_OBEX_HAS_NAME;
  flags.obex.accept = OPT_OBEX_HAS_ACCEPT;
  flags.obex.target = OPT_OBEX_TARGET_USER;
  flags.obex.ias = OPT_OBEX_DO_STDIAS;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    case 1:
      /* No argument : receive a beacon */
      break;
    default:
      /* Fallthrough */
    case 2:
      /* One argument : number of char to send */
      sscanf(argv[1], "%d", &n);
      fields.datalen = n + 1;
      fields.data = malloc(n + 1);
      memset(fields.data, 'J', n);
      fields.data[n] = '\0';
      fields.cleanup = OPT_DATA_DO_FREE;
      break;
    }

  if(argc > 1)
    {
      /* Send a single e-Squirt/Beacon and get associated interface */
      options.model = OPT_SQUIRT;	/* Make sure model is defined */
      ret = send_squirt(&fields, &options, &extra, &interface, &flags);

      /* Display it */
      debug_obget(&interface, &flags, &extra);

      if(ret < 0)
	printf("Sending e-Squirt failed...\n");

      /* Cleanup */
      free_structs(NULL, &extra, &interface);
    }
  else
    {
      /* Prepare to handle get interface calls */
      set_obget_handler(&reply_obget, NULL);

      /* Receive multiple e-Squirt/Beacon */
      printf("Waiting for e-Squirts over Obex...\n");
      recv_squirt(&fields, &options, &extra, &receive_esquirt, NULL);
      /* Will loop forever */
    }

  return(ret);
}
