/*
 *			    HTTP to e-Squirt proxy
 *			    ----------------------
 *
 *		Jean II '99, '00, '01 and '02
 *
 * Process HTTP requests from the CGI interface and convert them to e-Squirt
 * requests over IrDA.
 * 
 * This file is released under the GPL license.
 */

/*
 * CVS blurb
 * $Header$
 * $Log$
 */
/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include <squirt.h>

#define LOG_FILE

/************************** HTML MANGLING **************************/
/*
 * Processing of HTML pages. We mostly want to fix the embedded URLs
 * so that the other end does the right thing...
 */

/* ---------------------------------------------------------------- */
/*
 * Get path out of a name
 *
 * Just trucate everything after the last '/'
 */
static inline void
get_base_path(char *	basename)
{
  char *	p;
  char *	c;

  p = strrchr(basename, '/');
  if(p)
    {
      /* Check for special case : hostname only */
      c = strchr(basename, ':');
      if((c != NULL) && (p <= c+2))
	/* Don't remove hostname */
	;
      else
	/* Normal case, remove last component */
	*p = '\0';
    }
  else
    basename[0] = '\0';
}

/* ---------------------------------------------------------------- */
/*
 * Mangle a URL
 *
 * Add the "obex:/" prefix to each URL so that it goes back to us.
 * Also change relative URLs to absolute URLs.
 *
 * The source is a non-NULL terminated string.
 * Destination buffer plus its size.
 * Return lenght of new page.
 */
static char *
mangle_url(char *	d,		/* Destination */
	   char *	s,		/* Source */
	   char *	base)		/* Base URL (for relative URLs) */
{
  /* Only mangle the "obex:" URLs. They are the one that are supposed
   * to go back to us. Others, we don't care... */
  if(!strncasecmp(s, "obex:", 5))
    {
      strcpy(d, base);
      d += strlen(base);
      *d = '/';
      d++;
    }
  return(d);
}

/* ---------------------------------------------------------------- */
/*
 * Mangle an HTML page
 *
 * Check every URL and add the "obex:/" prefix so that it goes back to us.
 * Also change relative URLs to absolute URLs.
 * You can call this function with binary data, in 99.999% of cases it
 * won't touch it.
 *
 * The source is a non-NULL terminated string.
 * Destination buffer plus its size.
 * Return lenght of new page.
 */
static int
mangle_html(char *	dest,		/* Destination */
	    int		dlen,
	    char *	src,		/* Source */
	    int		slen,
	    char *	base)		/* Base URL (for relative URLs) */
{
  char *	s = src;
  char *	d = dest;
  char *	e = dest + dlen;
  char		last;

  /* Save last char of the source, replace with a NULL */
  last = src[slen - 1];
  src[slen - 1] = '\0';

  /* Look every character of the page */
  while((*s != '\0') && (d < e))
    {
      /* Is it a HTML tag ? */
      if(*s != '<')
	{
	  /* Just copy */
	  *d = *s;
	  d++;
	  s++;
	}
      else
	{
	  /* Copy */
	  *d = *s;
	  d++;
	  s++;

	  /* -------------- Check for an anchor -------------- */
	  if(((*s == 'a') || (*s == 'A')) && (isspace(s[1])))
	    {
	      /* Skip blanks (we should use isblank()) */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while(isspace(*s));

	      /* Check for href */
	      if(!strncasecmp(s, "href", 4))
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, base);
		}
	    }

	  /* -------------- Check for IMG tags -------------- */
	  if((!strncasecmp(s, "img", 3)) ||
	     (!strncasecmp(s, "script", 6)))
	    {
	      /* Go to source attribute */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while((*s != '>') && (strncasecmp(s, "src", 3)));

	      /* If we found a src attribute */
	      if(*s != '>')
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, base);
		}
	    }

	  /* -------------- Check for FORM tags -------------- */
	  if(!strncasecmp(s, "form", 4))
	    {
	      /* Go to source attribute */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while((*s != '>') && (strncasecmp(s, "action", 6)));

	      /* If we found a src attribute */
	      if(*s != '>')
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, base);
		}
	    }
	}
    }

  /* Check errors */
  if(*s == '\0')
    {
      int		l;

      /* Take care of binary data */
      l = (slen - 1) - (s - src);
      memcpy(d, s, l);			/* Or cable */
      d += l;
      /* And of last char */
      *d = last;
      d++;
      *d = '\0';
      return(d - dest);
    }
  else
    return(0);
}

/*************************** IP UTILITIES ***************************/
/*
 * Various TCP/IP helpers
 */

/*------------------------------------------------------------------*/
/*
 * Function get_local_ip()
 *
 *    Ask the resolver about our IP address...
 *
 * We need it, so we get it...
 * WARNING : getdomainname() return the NIS domain name, which we don't
 * care about...
 * Note : if the resolver is set appropriately, it should query the DNS
 * and default to the /etc/host file. And, most system configure their
 * own full domain name and IP address in this file (if they have one).
 * So, even if we are not connected to a network, this should work, and
 * we should fail only when no IP address and DNS name have been assigned
 * to this computer...
 */
int
get_local_ip(char *	buf,
	     int	buflen)
{
  struct hostent *	ptr_h;
  struct in_addr	in;
  char			dnsname[32];

  /* Get the host name first */
  if(gethostname(dnsname, sizeof(dnsname)) < 0)
    return(-1);
  dnsname[sizeof(dnsname)-1] = '\0';

  /* Get the host.domain name and IP address via the DNS resolver */
  if((ptr_h = gethostbyname(dnsname)) == NULL ||
     ptr_h->h_length != sizeof(in.s_addr))
    return(-1);

  /* Convert IP to a string */
  memcpy(&in.s_addr, ptr_h->h_addr, sizeof(in.s_addr));
  if(inet_ntop(AF_INET, &in, buf, buflen) == NULL)
    return(-1);

  return(0);
}


/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  FILE *	log;
  char *	cgi_query;
  char *	cgi_path;
  char *	cgi_method;
  char *	cgi_type;
  char *	cgi_length;
  char *	cgi_accept;
  char *	cgi_cookie;
  int		transport;
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	interface;
  gi_flags	flags;
  rui_data	html;		/* The RUI string */
  rui_params	params;		/* Processing opts */
  char *	postdata = NULL;
  int		postlen = 0;
  char		ip[32];
  char		url[128];
  char *	page;
  int		pagelen;
  int		ret;

  init_structs(&fields, &options, &extra, &interface, &flags, NULL);

  /* Get arguments passed to us by web server.
  * Those are passed as environment variables */
  cgi_query = getenv("QUERY_STRING");
  cgi_path = getenv("PATH_INFO");
  cgi_method = getenv("REQUEST_METHOD");
  cgi_type = getenv("CONTENT_TYPE");
  cgi_length = getenv("CONTENT_LENGTH");
  cgi_accept = getenv("HTTP_ACCEPT");
  cgi_cookie = getenv("HTTP_COOKIE");

#ifdef LOG_FILE
  /* Put them in the log */
  log = fopen("/tmp/s2_obex_cgi.log", "w+");
  if(log != NULL)
    {
      fprintf(log, "Starting\n");

      if(cgi_query != NULL)
	fprintf(log, "Query [%s]\n", cgi_query);
      if(cgi_path != NULL)
	fprintf(log, "Path [%s]\n", cgi_path);
      if(cgi_method != NULL)
	fprintf(log, "Method [%s]\n", cgi_method);
      if(cgi_type != NULL)
	fprintf(log, "Type [%s]\n", cgi_type);
      if(cgi_length != NULL)
	fprintf(log, "Length [%s]\n", cgi_length);
      if(cgi_accept != NULL)
	fprintf(log, "Accept [%s]\n", cgi_accept);
      if(cgi_cookie != NULL)
	fprintf(log, "Cookie [%s]\n", cgi_cookie);
      fflush(log);
    }
#endif	/* LOG_FILE */

  /* Find if it's a BlueTooth request or an IrDA request */
  /* Get rid of initial '/' if we have a full form */
  if(!strncasecmp(cgi_path, "/bt:", 4))
    cgi_path++;
  if(!strncasecmp(cgi_path, "bt:", 3))
    {
      char *	ptr;
      int	len;

      /* Find end of BlueTooth prefix */
      ptr = strchr(cgi_path, '/');
      len = ptr - cgi_path - 3;

      /* Check validity */
      if((ptr == NULL) || (len > 18))
	{
#ifdef LOG_FILE
	  fprintf(log, "Invalid BT prefix\n");
#endif	/* LOG_FILE */
	  return(-1);
	}

      /* Check is there is an address or none */
      if(len > 0)
	{
	  /* Copy link address */
	  memcpy(extra.linkaddr, cgi_path + 3, len);
	  extra.linkaddr[len] = '\0';
	  flags.linkaddr = OPT_DO_LINKADDR;
	  options.linkaddr = OPT_DO_LINKADDR;

#ifdef LOG_FILE
	  if(log != NULL)
	    fprintf(log, "BT address [%s]\n", extra.linkaddr);
#endif	/* LOG_FILE */
	}

      /* Skip prefix */
      cgi_path = ptr;

      transport = OPT_BLUETOOTH;
    }
  else
    {
      //flags->linkaddr = OPT_NO_LINKADDR;
      transport = OPT_IRDA;
    }

  /* Make sure that the path is defined */
  if((cgi_path == NULL) || (cgi_path[0] == '\0'))
    cgi_path = "/";

  /* Get rid of initial '/' if we have a full form */
  if(!strncasecmp(cgi_path, "/obex:", 6))
    cgi_path++;

#ifdef LOG_FILE
  if(log != NULL)
    fprintf(log, "Reduced path [%s]\n", cgi_path);
#endif	/* LOG_FILE */

  /*
   * TODO :
   * o Check method, do processing as required
   */

  /* Re-Construct the URL */
  params.browser = OPT_RUI_BROWSER_NONE;
  params.form    = OPT_RUI_FORM_NONE;
  params.prefix  = OPT_RUI_PREFIX_OBEX;
  html.action = cgi_path;
  html.input = cgi_query;

  /* Check the type of method */
  if(!strcasecmp(cgi_method, "GET"))
    {
      /* Get might be a file or a form */
      if((cgi_query) && (*cgi_query))
	params.form = OPT_RUI_FORM_GET;
      else
	params.form = OPT_RUI_FORM_NONE;
    }
  else
    if(!strcasecmp(cgi_method, "POST"))
      {
	params.form = OPT_RUI_FORM_POST;
	html.encoding = cgi_type;
	/* Get POST data */
	if((cgi_length == NULL) || (sscanf(cgi_length, "%d", &postlen) != 1))
	  return(-1);
	postdata = malloc(postlen + 1);
	if(!postdata)
	  return(-1);
	fread(postdata, 1, postlen, stdin);
	postdata[postlen] = '\0';
	html.input = postdata;
      }
    else
      return(-1);

  /* Create e-Squirt request */
  if(rui_parse_action(&html, &params,
		      &fields, &options, &extra, &interface, &flags) < 0)
    return(-1);

  /* Cleanup */
  if(postdata)
    free(postdata);

  /* Put cookie in the HTTP-Header.
   * If we wanted to add Accept, this is the place ;-) */
  if(cgi_cookie)
    {
      int	len = strlen(cgi_cookie);

      /* Create HTTP-Header */
      extra.http_hdr = malloc(len + 32);
      extra.http_cleanup = OPT_DATA_DO_FREE;
      flags.obex.http = OPT_OBEX_HAS_HTTP;

      /* Put cookie in it */
      sprintf(extra.http_hdr, "Cookie:%s\r\n", cgi_cookie);
    }

  /* Fix up transport */
  options.transport = transport;
  flags.transport = transport;
 
  /* A few extra options... */
  options.obex.target = OPT_OBEX_TARGET_UUGLI;
  //options.obex.ias = OPT_OBEX_DO_STDIAS;
  //flags.obex.ias = OPT_OBEX_DO_STDIAS;
  // Errors don't come properly if we enable STDIAS, because after
  // receiving an error for the remote server, the library will retry
  // on the STDIAS and it will fail, so we won't get the proper error
  // code. Maybe I should change the behavior of the library, but I'm
  // not 100% sure.
  // For now, disable STDIAS.
  options.obex.ias = OPT_OBEX_NO_STDIAS;
  flags.obex.ias = OPT_OBEX_NO_STDIAS;

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "URL [%s]\n", interface.data);
      fflush(log);
    }
#endif	/* LOG_FILE */

  /* Send a single e-Squirt/Beacon and get associated interface */
  ret = send_squirt(&fields, &options, &extra, &interface, &flags);
  if(ret < 0)
    {
      /* Cleanup */
      free_structs(NULL, &extra, &interface);

      /* Here, we have two types of return code.
       * -1 -> -9 are internal failures to e-Squirt library
       * < -10 are OBEX failures codes returned from the Obex server.
       * We need to translate them to proper HTTP failure codes */
      if(ret < -10)
	{
	  int	obex_ret = -ret;
#ifdef LOG_FILE
	  if(log != NULL)
	    {
	      fprintf(log, "OBEX error %X -> %d0%d\n",
		      obex_ret, obex_ret >> 4, obex_ret & 0xF);
	      fclose(log);
	    }
#endif	/* LOG_FILE */

	  printf("Status: %d0%d\r\n", obex_ret >> 4, obex_ret & 0xF);
	  return(0);
	}
      return(-1);
    }

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "datalen %d\n", interface.datalen);
      fflush(log);
    }
#endif	/* LOG_FILE */

  /* Get our IP address, fallback on 127.0.0.1.
   * We try to avoid 127.0.0.1 because the web server may not be
   * co-located on it (and also to defeat proxy stuff). - Jean II */
  if(get_local_ip(ip, sizeof(ip)) < 0)
    strcpy(ip, "127.0.0.1");

  /* Create base URL refering to us */
  if(transport == OPT_BLUETOOTH)
    {
      /* The library automatically return the link address, so we can
       * refer link to their proper BlueTooth originator. */
      if(flags.linkaddr == OPT_DO_LINKADDR)
	sprintf(url, "http://%s/obex/bt:%s", ip, extra.linkaddr);
      else
	sprintf(url, "http://%s/obex/bt:", ip);
    }
  else
    sprintf(url, "http://%s/obex", ip);

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "Prefix [%s]\n", url);
      if(flags.obex.http == OPT_OBEX_HAS_HTTP)
	fprintf(log, "HTTP [%s]\n", extra.http_hdr);

      fprintf(log, "Ending\n");
      fclose(log);
    }
#endif	/* LOG_FILE */

  /* Create space for the data */
  page = malloc(interface.datalen * 2 + 256);
  if(!page)
    return(-1);

  /* Mangle the HTML page */
  pagelen = mangle_html(page, interface.datalen * 2 + 255,
			interface.data, interface.datalen,
			url);

#if 0
  /* Super hack to test cookies. Huge privacy risk as it is...
   * Code known to be broken in most cases, works only with www.google.com.
   * Beware ! Jean II */
  if(flags.obex.http == OPT_OBEX_HAS_HTTP)
    {
      int		len = strlen(extra.http_hdr);
      char *	cookies = malloc(len + 16);
      char *	pos;
      memcpy(cookies, extra.http_hdr, len + 1);
      pos = strstr(cookies, "domain=");
      if(pos)
	{
	  sprintf(pos + 7, "127.0.0.1\r\n");
	}
      fwrite(cookies, 1, strlen(cookies), stdout);
    }
#endif

  /* Return everything to the web server */
  printf("Content-type: %s\r\n\r\n", extra.type);
  fwrite(page, 1, pagelen, stdout);

  /* Cleanup */
  free(page);
  free_structs(NULL, &extra, &interface);

  return(0);
}
