/*
 *			    e-Squirt web service
 *			    --------------------
 *
 *		Jean II '99, '00, '01 and '02
 *
 * Process e-Squirt encapsulated in HTTP requests from the CGI interface.
 * 
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include <squirt_xml.h>
#include <squirt_rui.h>

#define LOG_FILE

//#define BROWSER		"netscape -remote '%s' -display ':0'"
//#define BROWSER		"mozilla -remote '%s' --display=':0'"
//#define BROWSER		"~/mozilla/netscape-remote -remote '%s'"
//#define BROWSER		"mozilla -remote '%s' --display='bougret:0'"
//#define BROWSER	"/usr/bin/X11/netscape -remote '%s' -display 'bougret:0'"
//#define BROWSER	"/usr/bin/X11/netscape -remote '%s' -display ':0'"
//#define BROWSER		"/home/jean/mozilla/mozilla/mozilla -remote '%s' --display=':0'"
#define BROWSER		"~/cgi-bin/netscape-remote -remote '%s' --display=':0'"

typedef struct command_binding
{
  char *	es_name;
  char *	ns_name;
  int		repeat;
} command_binding;

#define OPENURL		"openURL(%s)"
const command_binding	commands[] =
{
  { "cmd:up",		"PageUp()",		1 },
  { "cmd:down",		"PageDown()",		1 },
  { "cmd:lup",		"LineUp()",		1 },
  { "cmd:ldown",	"LineDown()",		1 },
  { "cmd:6up",		"LineUp()",		6 },
  { "cmd:6down",	"LineDown()",		6 },
  { "cmd:back",		"back()",		1 },
  { "cmd:forward",	"forward()",		1 },
  { "cmd:left",		"ColumnLeft()",		1 },
  { "cmd:right",	"ColumnRight()",	1 },
  { "cmd:6left",	"ColumnLeft()",		6 },
  { "cmd:6right",	"ColumnRight()",	6 },
};
#define NUM_COMMANDS	(sizeof(commands)/sizeof(command_binding))

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon.
 */
int
call_webbrowser(char *		url)
{
  unsigned char	bufn[550];	/* The netscape command */
  unsigned char	bufs[575];	/* The system command */
  int		repeat = 1;
  int		i;

  /* By default, open the URL */
  sprintf(bufn, OPENURL, url);
  /* Check if it's one of the known command */
  for(i = 0; i < NUM_COMMANDS; i++)
    if(!strcmp(url, commands[i].es_name))
      {
	strcpy(bufn, commands[i].ns_name);
	repeat = commands[i].repeat;
	break;
      }
  /* Just do it */
  sprintf(bufs, BROWSER, bufn);
  fprintf(stderr, "Running command [%s]\n", bufs);
  for(i = 0; i < repeat; i++)
    system(bufs);

  /* Return back to the server */
  return(0);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  FILE *	log;
  char *	cgi_query;
  //char *	cgi_path;
  char *	cgi_method;
  //char *	cgi_type;
  char *	cgi_length;
  es_squirt	fields;
  es_opt	options;
  char		xml[ES_MAX_XML];
  int		postlen = 0;
  int		ret;

  init_structs(&fields, &options, NULL, NULL, NULL, NULL);
  fields.data = xml;

  /* Get arguments passed to us by web server.
  * Those are passed as environment variables */
  cgi_query = getenv("QUERY_STRING");
  //cgi_path = getenv("PATH_INFO");
  cgi_method = getenv("REQUEST_METHOD");
  //cgi_type = getenv("CONTENT_TYPE");
  cgi_length = getenv("CONTENT_LENGTH");

#ifdef LOG_FILE
  /* Put them in the log */
  log = fopen("/tmp/squirt_cgi.log", "w+");
  if(log != NULL)
    {
      fprintf(log, "Starting\n");

      if(cgi_query != NULL)
	fprintf(log, "Query [%s]\n", cgi_query);
      //if(cgi_path != NULL)
      //fprintf(log, "Path [%s]\n", cgi_path);
      if(cgi_method != NULL)
	fprintf(log, "Method [%s]\n", cgi_method);
      //if(cgi_type != NULL)
      //fprintf(log, "Type [%s]\n", cgi_type);
      if(cgi_length != NULL)
	fprintf(log, "Length [%s]\n", cgi_length);
      fflush(log);
    }
#endif	/* LOG_FILE */

  /* Get XML string
   * Check the type of method to find where are the arguments */
  if(!strcasecmp(cgi_method, "GET"))
    {
      char *	ref;
      int	len;

      /* Get might be a file or a form */
      if((cgi_query == NULL) || (*cgi_query == '\0'))
	/* Could do better -> Not accepted or whatever*/
	return(-1);
      /* For GET, the reference is in the Query, prefixed by "reference=" */
      ref = strchr(cgi_query, '=');
      if((strncasecmp(cgi_query, "reference", 9)) ||
	 (ref == NULL))
	return(-1);
      /* Skip whitespaces */
      do
	ref++;
      while(isspace(*ref));
      len = strlen(ref);
      if(len > ES_MAX_XML)
	return(-1);
      unescape_uri(fields.data, ref);
    }
  else
    if(!strcasecmp(cgi_method, "POST"))
      {
	/* Check POST data */
	if((cgi_length == NULL) ||
	   (sscanf(cgi_length, "%d", &postlen) != 1) ||
	   (postlen > ES_MAX_XML))
	  return(-1);
	/* Get POST data :
	 * For POST, the reference is directly in the body */
	fread(fields.data, 1, postlen, stdin);
	fields.data[postlen] = '\0';
	unescape_uri(fields.data, fields.data);
      }
    else
      return(-1);

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "xml [%s]\n", fields.data);
    }
#endif	/* LOG_FILE */

  /* Parse the XML string */
  ret = parse_xml(&fields, &options);
  if(ret < 0)
      return(-1);

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "label [%s]\n", fields.label);
      fprintf(log, "url [%s]\n", fields.url);
      fprintf(log, "type [%s]\n", fields.type);

      fprintf(log, "Ending\n");
      fclose(log);
    }
#endif	/* LOG_FILE */
    
  /* Call web browser */
  call_webbrowser(fields.url);

  /* Return something to the web server */
  printf("Content-type: text/html\r\n\r\nDone");

  return(0);
}
