/*
 *			    Obex PUT web service
 *			    --------------------
 *
 *		Jean II '99, '00, '01 and '02
 *
 * Process OBEX PUT encapsulated in HTTP requests from the CGI interface.
 * 
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#define LOG_FILE

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  FILE *	log;
  char *	cgi_query;
  char *	cgi_path;
  char *	cgi_method;
  char *	cgi_type;
  char *	cgi_length;
  char *	data;
  int		datalen;
  FILE *	output;

  /* Get arguments passed to us by web server.
  * Those are passed as environment variables */
  cgi_query = getenv("QUERY_STRING");
  cgi_path = getenv("PATH_INFO");
  cgi_method = getenv("REQUEST_METHOD");
  cgi_type = getenv("CONTENT_TYPE");
  cgi_length = getenv("CONTENT_LENGTH");

#ifdef LOG_FILE
  /* Put them in the log */
  log = fopen("/tmp/obexput_cgi.log", "w+");
  if(log != NULL)
    {
      fprintf(log, "Starting\n");

      if(cgi_query != NULL)
	fprintf(log, "Query [%s]\n", cgi_query);
      if(cgi_path != NULL)
	fprintf(log, "Path [%s]\n", cgi_path);
      if(cgi_method != NULL)
	fprintf(log, "Method [%s]\n", cgi_method);
      if(cgi_type != NULL)
	fprintf(log, "Type [%s]\n", cgi_type);
      if(cgi_length != NULL)
	fprintf(log, "Length [%s]\n", cgi_length);
      fflush(log);
    }
#endif	/* LOG_FILE */

  /* Check the type of method to find where are the arguments */
  if(!strcasecmp(cgi_method, "GET"))
    {
      /* We don't support the GET method */
      return(-1);
    }
  else
    if(!strcasecmp(cgi_method, "POST"))
      {
	/* Check POST data */
	if((cgi_length == NULL) ||
	   (sscanf(cgi_length, "%d", &datalen) != 1))
	  return(-1);
	/* Get POST data :
	 * For POST, the data is directly in the body */
	/* Allocate */
	data = malloc(datalen);
	if(!data)
	  return(-2);
	/* Suck the data */
	fread(data, 1, datalen, stdin);
      }
    else
      return(-1);

#ifdef LOG_FILE
  if(log != NULL)
    {
      fprintf(log, "datalen [%d]\n", datalen);

      fprintf(log, "Ending\n");
      fclose(log);
    }
#endif	/* LOG_FILE */

  /* Open output file */
  output = fopen("/tmp/obexput.out", "w+");
  if(output == NULL)
    return(-3);

  /* Write the file */
  fwrite(data, 1, datalen, output);

  /* Close */
  fclose(output);

  /* Return something to the web server */
  printf("Content-type: text/html\r\n\r\nDone");

  return(0);
}
