/*
 *			    Push a file over HTTP
 *			    ---------------------
 *
 *		Jean II '99 & '00 & '03
 *
 * This file is released under the GPL license.
 */

/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <ctype.h>

#include <squirt.h>
#include <http.h>

/**************************** CONSTANTS ****************************/

const char	esurl[] = "http://127.0.0.1/esquirt";
const char	opurl[] = "http://127.0.0.1/obexput/";

/************************* FILE SUBROUTINES *************************/
/*
 * Extracted from obex_io.c, (c) Pontus Fuchs <pontus.fuchs@tactel.se>
 */

/* ---------------------------------------------------------------- */
/*
 * Get the filesize in a "portable" way
 */
static inline int
get_filesize(const char *	filename)
{
  struct stat stats;

  /*  Need to know the file length */
  stat(filename, &stats);

  return (int) stats.st_size;
}

/* ---------------------------------------------------------------- */
/*
 * Read a file and alloc a buffer for it
 */
static char *
easy_readfile(const char *	filename,
	      int *		file_size)
{
  int		actual;
  int		fd;
  char *	buf;

  *file_size = get_filesize(filename);
  printf("name=%s, size=%d\n", filename, *file_size);

  fd = open(filename, O_RDONLY, 0);
  if(fd == -1)
    return(NULL);

  buf = malloc(*file_size);
  if(buf == NULL)
    return(NULL);

  actual = read(fd, buf, *file_size);
  close(fd); 

  *file_size = actual;

  return(buf);
}

/************************* RANDOM UTILITIES *************************/

/* ---------------------------------------------------------------- */
/*
 * Initialise the structures with reasonable values
 */
void
init_structs(es_squirt *	content,
	     es_opt *		options,
	     es_trans *		extra,
	     gi_obget *		request,
	     gi_flags *		flags,
	     es_server_data *	control)
{
  /* The es_squirt structure */
  if(content != NULL)
    memset(content, 0, sizeof(es_squirt));
  /* The es_opt structure */
  if(options != NULL)
    {
      memset(options, 0, sizeof(es_opt));
      options->xml = OPT_XML_STD;
    }
  /* The gi_obget structure */
  if(request != NULL)
    {
      memset(request, 0, sizeof(gi_obget));
      request->datalen = -1;
    }
  /* The gi_opt structure */
  if(flags != NULL)
    memset(flags, 0, sizeof(gi_flags));
  /* The es_trans structure */
  if(extra != NULL)
    {
      memset(extra, 0, sizeof(es_trans));
      extra->targetlen = -1;
    }
  /* The es_server_data structure */
  if(control != NULL)
    memset(control, 0, sizeof(es_server_data));
}

/* ---------------------------------------------------------------- */
/*
 * Reclaim memory allocated in structures
 */
void
free_structs(es_squirt *	content,
	     es_trans *		extra,
	     gi_obget *		request)
{
  /* The es_squirt structure */
  if(content != NULL)
    {
      /* Get rid of potential squirt data passed to us */
      if((content->data != NULL) && (content->cleanup == OPT_DATA_DO_FREE))
	{
	  free(content->data);
	  content->data = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(content->data)\n", __FUNCTION__);
#endif
	}
    }

  /* The es_trans structure */
  if(extra != NULL)
    {
      /* Get rid of potential http header passed to us */
      if((extra->http_hdr != NULL) &&
	 (extra->http_cleanup == OPT_DATA_DO_FREE))
	{
	  free(extra->http_hdr);
	  extra->http_hdr = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(extra->http_hdr)\n", __FUNCTION__);
#endif
	}
    }

  /* The gi_obget structure */
  if(request != NULL)
    {
      /* Get rid of potential form data passed to us */
      if((request->data != NULL) && (request->cleanup == OPT_DATA_DO_FREE))
	{
	  free(request->data);
	  request->data = NULL;
#ifdef DEBUG_ALLOC
	  fprintf(stderr, "%s: free(request->data)\n", __FUNCTION__);
#endif
	}
    }
}

/************************** HTML MANGLING **************************/
/*
 * Processing of HTML pages. We mostly want to fix the embedded URLs
 * so that the other end does the right thing...
 */

/* ---------------------------------------------------------------- */
/*
 * Get path out of a name
 *
 * Just truncate everything after the last '/'
 * Also return the hostname
 */
static inline void
get_base_path(char *	basename,
	      char *	hostname)
{
  char *	p;
  int		l = strlen(basename);
  char *	e;

  /* Check if name contains the host name */ 
  p = strstr(basename, "://");
  if(p)
    {
      /* Find end of hostname */
      p += 3;
      p = strchr(p, '/');
      if(p)
	l = p - basename;
      else
	p = basename + l;
      memcpy(hostname, basename, l);
      hostname[l] = '\0';
    }
  else
    {
      hostname[0] = '\0';
      p = basename;
    }

  /* Find end of the path */
  e = strrchr(p, '/');
  if(!e)
    e = p;

  /* Remove last component */
  *e = '\0';
}

/* ---------------------------------------------------------------- */
/*
 * Mangle a URL
 *
 * Add the "obex:/" prefix to each URL so that it goes back to us.
 * Also change relative URLs to absolute URLs.
 *
 * The source is a non-NULL terminated string.
 * Destination buffer plus its size.
 * Return lenght of new page.
 */
static char *
mangle_url(char *	d,		/* Destination */
	   char *	s,		/* Source */
	   char *	path,		/* Base URL (for relative URLs) */
	   char *	host)		/* Host name (for relative URLs) */
{
  /* Check if the URL is not relative to the page itself */
  if(*s != '#')
    {
      /* Absolute URLs */
      if(!strncasecmp(s, "http:", 5))
	{
	  /* Insert "obex:/" prefix so that we can trap it */
	  memcpy(d, "obex:/", 6);
	  d += 6;
	}
      else
	{
	  char *q = strchr(s, '"');
	  char *c = strchr(s, ':');

	  if((c != NULL) && (c < q))
	    {
	      /* Other protocol, such as "ftp:", "mailto:" :
	       * We don't support it, so don't touch... */
	      ;
	    }
	  else
	    {
	      if(*s == '/')
		{
		  /* URL relative to the current server, make it absolute. */
		  memcpy(d, "obex:/", 6);
		  d += 6;
		  strcpy(d, host);
		  d += strlen(host);
		}
	      else
		{
		  /* URL relative to the current path, make it absolute. */
		  memcpy(d, "obex:/", 6);
		  d += 6;
		  strcpy(d, path);
		  d += strlen(path);
		  *d = '/';
		  d++;
		}
	    }
	}
      /* Finito ! */
    }
  return(d);
}

/* ---------------------------------------------------------------- */
/*
 * Mangle an HTML page
 *
 * Check every URL and add the "obex:/" prefix so that it goes back to us.
 * Also change relative URLs to absolute URLs.
 * You can call this function with binary data, in 99.999% of cases it
 * won't touch it.
 *
 * The source is a non-NULL terminated string.
 * Destination buffer plus its size.
 * Return lenght of new page.
 */
static int
mangle_html(char *	dest,		/* Destination */
	    int		dlen,
	    char *	src,		/* Source */
	    int		slen,
	    char *	path,		/* Base URL (for relative URLs) */
	   char *	host)		/* Host name (for relative URLs) */
{
  char *	s = src;
  char *	d = dest;
  char *	e = dest + dlen;
  char		last;

  /* Save last char of the source, replace with a NULL */
  last = src[slen - 1];
  src[slen - 1] = '\0';

  /* Look every character of the page */
  while((*s != '\0') && (d < e))
    {
      /* Is it a HTML tag ? */
      if(*s != '<')
	{
	  /* Just copy */
	  *d = *s;
	  d++;
	  s++;
	}
      else
	{
	  /* Copy */
	  *d = *s;
	  d++;
	  s++;

	  /* -------------- Check for an anchor -------------- */
	  if(((*s == 'a') || (*s == 'A')) && (isspace(s[1])))
	    {
	      /* Skip blanks (we should use isblank()) */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while(isspace(*s));

	      /* Check for href */
	      if(!strncasecmp(s, "href", 4))
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, path, host);
		}
	    }

	  /* -------------- Check for IMG and SCRIPT tags -------------- */
	  if((!strncasecmp(s, "img", 3)) ||
	     (!strncasecmp(s, "script", 6)))
	    {
	      /* Go to source attribute */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while((*s != '>') && (strncasecmp(s, "src", 3)));

	      /* If we found a src attribute */
	      if(*s != '>')
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, path, host);
		}
	    }

	  /* -------------- Check for FORM tags -------------- */
	  if(!strncasecmp(s, "form", 4))
	    {
	      /* Go to source attribute */
	      do
		{
		  *d = *s;
		  d++;
		  s++;
		}
	      while((*s != '>') && (strncasecmp(s, "action", 6)));

	      /* If we found a src attribute */
	      if(*s != '>')
		{
		  /* Move to the URL */
		  do
		    {
		      *d = *s;
		      d++;
		      s++;
		    }
		  while(*s && (*s != '>') && (*(s - 1) != '"'));

		  if(*(s - 1) == '"')
		    d = mangle_url(d, s, path, host);
		}
	    }
	}
    }

  /* Check errors */
  if(*s == '\0')
    {
      int		l;

      /* Take care of binary data */
      l = (slen - 1) - (s - src);
      memcpy(d, s, l);			/* Or cable */
      d += l;
      /* And of last char */
      *d = last;
      d++;
      *d = '\0';
      return(d - dest);
    }
  else
    return(0);
}

/***************************** RECEIVER *****************************/

/* ---------------------------------------------------------------- */
/*
 * Receive a e-Squirt of Beacon. Non forking.
 */
int
receive_esquirt(es_squirt *	fields,		/* Rx payload */
		es_opt *	options,	/* Rx options */
		es_trans *	extra,		/* Rx transport data */
		gi_obget *	ret_hdr,	/* Not used */
		gi_flags *	ret_flags)	/* Not used */
{
  int		ret = 0;	/* Reply Header not set */
  char *	url;		/* URL to query */
  int		urllen;		/* Length of URL */
  char		hdr[256];	/* POST header to send */
  HTTP_Extra	http;		/* HTTP header + POST data */
  HTTP_Response reply;		/* HTTP reply */
  const char	hdrstart[] = "Content-Type: ";
  int		startlen;

  // Binary == yuck...
  //debug_squirt(fields, options, extra);

  memset(&http, '\0', sizeof(http));

  /* Deal with the type */
  if((options->obex.type != OPT_UNDEFINED) &&
     (options->obex.type != OPT_OBEX_TYPE_NONE))
    {
      /* Obex type is alway written in extra->type */
      startlen = strlen(hdrstart);
      memcpy(hdr, hdrstart, startlen);
      strcpy(hdr + startlen, extra->type);
      http.Headers = hdr;
    }
  else
    {
      /* Empty header */
      hdr[0] = '\0';
    }

  /* Check if it's a standard OBEX PUT and not a reference */
  if((options->protocol == OPT_OBEX) &&
     //(options->obex.name == OPT_OBEX_HAS_NAME) &&
     !(options->obex.type & (OPT_OBEX_TYPE_REFS |
			     OPT_OBEX_TYPE_REFX |
			     OPT_OBEX_TYPE_REFL)) )
    {
      /* Create Obex PUT request (as a POST) */
      startlen = strlen(opurl);

      /* Check if there is a name */
      if(options->obex.name == OPT_OBEX_HAS_NAME)
	urllen = startlen + strlen(extra->filename);
      else
	urllen = startlen;

      url = malloc(urllen + 1);
      memcpy(url, opurl, startlen + 1);

      /* Add filename as needed, after the '/' */
      if(options->obex.name == OPT_OBEX_HAS_NAME)
	memcpy(url + startlen, extra->filename,
	       strlen(extra->filename) + 1);

      /* Add a X-Linkname header as needed */
      if(options->linkaddr == OPT_DO_LINKADDR)
	{
	  startlen = strlen(hdr);
	  /* Add separator if already a type */
	  if(startlen)
	    {
	      strcpy(hdr + startlen, "\r\n");
	      startlen += 2;
	    }
	  /* Add linkname */
	  sprintf(hdr + startlen, "X-Linkname: %s", extra->linkaddr);
	  http.Headers = hdr;
	}
    }
  else
    {
      /* Create e-Squirt v3 request (as a POST) */
      urllen = strlen(esurl);
      url = malloc(urllen + 1);
      memcpy(url, esurl, urllen + 1);
    }

  /* Post data go there */
  http.PostData = fields->data;
  http.PostLen = fields->datalen;
 
  fprintf(stderr, "URL = [%s] (%d)\n", url, urllen);
  printf("Encoding : [%s]\n", hdr);

  /* Try to fetch the page - local : never use proxy */
  reply = http_request(url, &http, kHMethodPost,
		       HFLAG_RETURN_HEADER | HFLAG_POST_USER_TYPE |
		       HFLAG_FORCE_NO_PROXY);
  /* No longer needed */
  free(url);

  /* Check if we can return some data */
  if(ret_hdr != (gi_obget *) NULL)
    {
      /* Check if TCP did fail */
      if((reply.lSize <= 0) || (reply.pData == NULL))
	{
	  if(reply.pHdr != NULL)
	    free(reply.pHdr);
	  /* Return standard error to server -> will send back NOT_FOUND */
	  return(-1);
	}

      /* Check if HTTP did fail */
      if(strcmp(reply.szHCode, "200"))
	{
	  int	obex_err;

	  /* Cleanup */
	  if(reply.pHdr != NULL)
	    free(reply.pHdr);
	  free(reply.pData);

	  /* Compute Obex error code and return it*/
	  obex_err = ((reply.szHCode[0] - '0') << 4) | (reply.szHCode[2] - '0');
	  printf("Error : [%s] -> %X\n", reply.szHCode, obex_err);
	  return(-obex_err);
	}

      /* Set the name */
      ret_flags->obex.name = OPT_OBEX_NO_NAME;
      /* Set the type */
      ret_flags->obex.type = OPT_OBEX_TYPE_HTML;
      if(reply.pHdr != NULL)
	{
	  //printf("Some headers\n");
	  if(find_header(reply.pHdr, 0, "Content-Type:",
			 extra->type, sizeof(extra->type)))
	    ret_flags->obex.type = OPT_OBEX_TYPE_USER;
	  free(reply.pHdr);
	}

      /* Post process the HTML */

      /* Create space for the data */
      ret_hdr->data = malloc(reply.lSize * 2 + 256);
      if(!ret_hdr->data)
	{
	  free(reply.pData);
	  return(-1);
	}

      /* Mangle the HTML page */
      ret_hdr->datalen = mangle_html(ret_hdr->data, reply.lSize * 2 + 255,
				     reply.pData, reply.lSize,
				     "", "");

      /* Cleanup */
      ret_hdr->cleanup = OPT_DATA_DO_FREE;
      free(reply.pData);

      ret = 1;	/* Reply Header set */
    }
  else
    {
      /* Cleanup */
      if(reply.pHdr != NULL)
	free(reply.pHdr);
      if(reply.pData != NULL)
	free(reply.pData);
      ret = 0;		/* e-Squirt accepted, Reply Header not set */
    }

  /* Return back to the server */
  return(ret);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  es_trans	extra;
  gi_obget	ret_hdr;
  gi_flags	ret_flags;
  int		ret = -1;

  init_structs(&fields, &options, &extra, &ret_hdr, &ret_flags, NULL);
  /* Set the options */
  options.do_xml = OPT_NO_XML;	/* Don't handle XML in the lib */
  options.transport = OPT_BLUETOOTH | OPT_IRDA;
  options.protocol = OPT_OBEX;
  options.obex.type = OPT_OBEX_TYPE_NONE;
  options.obex.name = OPT_OBEX_HAS_NAME;
  options.obex.session = OPT_OBEX_DO_SESSION;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    default:
      /* Fallthrough */
    case 1:
      /* No argument : error */
      printf("Error : %s filename type linkaddr\n", argv[0]);
      return(-1);

    case 4:
      /* Third argument : Link Address */
      strcpy(extra.linkaddr, argv[3]);
      options.linkaddr = OPT_DO_LINKADDR;
      options.transport = OPT_BLUETOOTH;

      /* Fallthrough */
    case 3:
      /* Second argument : Mime Type */
      strcpy(extra.type, argv[2]);
      options.obex.type = OPT_OBEX_TYPE_USER;

      /* Fallthrough */
    case 2:
      /* One argument : the filename */
      strcpy(extra.filename, argv[1]);
      options.obex.name = OPT_OBEX_HAS_NAME;

      break;
    }

  /* Allocate and read the file */
  fields.data = easy_readfile(extra.filename, &fields.datalen);
  fields.cleanup = OPT_DATA_DO_FREE;
  if(fields.data == NULL)
    {
      printf("Reading file failed...\n");
      return(-1);
    }

  /* Send the file */
  ret = receive_esquirt(&fields, &options, &extra, &ret_hdr, &ret_flags);
  if(ret < 0)
    printf("Sending file failed (%d)...\n", ret);
  else
    if(ret > 0)
      {
	/* Cleanup */
	free_structs(NULL, &extra, &ret_hdr);
      }

  return(ret);
}
