/*
 *			    e-Squirt v3 client sample
 *			    -------------------------
 *
 *		Jean II '99, '00, '01 and '02
 *
 * Listen to e-Squirt requests over IrDA and convert them to HTTP requests.
 * 
 * This file is released under the GPL license.
 */

/*
 * CVS blurb
 * $Header$
 * $Log$
 */
/*
Copyright (C) 1995-2001 Hewlett-Packard Company 

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details. 

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
*/

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <squirt_xml.h>
#include <squirt_rui.h>

#include <http.h>

/****************************** SENDER ******************************/

/* ---------------------------------------------------------------- */
/*
 * Send an e-Squirt v3 via a POST
 */
int
send_httppost(es_squirt *	fields,		/* Payload */
	      es_opt *		options)	/* Options */
{
  int		ret = 0;	/* Reply Header not set */
  char		url[] = "http://any.adhoc/esquirt";
  char *	hdr = "";			/* POST header to send */
  HTTP_Response reply;				/* HTTP reply */
  HTTP_Extra	http;			/* HTTP header + POST data */

  memset(&http, '\0', sizeof(http));

  /* Post data go there */
  http.PostData = fields->data;
  http.PostLen = fields->datalen;
 
  /* Take care of type */
  if(fields->data[0] == '<')
    hdr = "Content-Type: text/x-ref-xml";
  else
    hdr = "Content-Type: text/x-ref-simple";
  http.Headers = hdr;

  //fprintf(stderr, "URL = [%s]\n", url);
  //printf("Encoding : [%s]\n", hdr);

  /* Try to fetch the page */
  reply = http_request(url, &http, kHMethodPost,
		       HFLAG_RETURN_HEADER | HFLAG_POST_USER_TYPE |
		       HFLAG_FORCE_NO_PROXY);

  /* We don't really care about returned data */

  /* Cleanup */
  if(reply.pHdr != NULL)
    free(reply.pHdr);
  if(reply.pData != NULL)
    free(reply.pData);
  ret = 0;

  return(ret);
}

/******************************* MAIN *******************************/

/* ---------------------------------------------------------------- */
/*
 * main body of the program
 */
int
main(int	argc,
     char *	argv[])
{
  es_squirt	fields;
  es_opt	options;
  char		xml[ES_MAX_XML];
  int		ret = -1;

  init_structs(&fields, &options, NULL, NULL, NULL, NULL);
  fields.data = xml;
  fields.datalen = ES_MAX_XML;

  /* Set the receive options that matter */
  options.xml = OPT_XML_STD;

  /* Look if there is any command line args */
  switch(argc)
    {
    case 0:
      /* Fallthrough */
    case 1:
      /* No argument : error */
      fprintf(stderr, "esquirt3 label url type\n");
      return(-1);
      break;
    case 2:
      /* One argument : the URL */
      strcpy(fields.url, argv[1]);
      break;
    case 3:
      /* Two arguments : label + url */
      strcpy(fields.label, argv[1]);
      strcpy(fields.url, argv[2]);
      break;
    default:
      /* Fallthrough */
    case 4:
      /* Three arguments : label + url + type */
      strcpy(fields.label, argv[1]);
      strcpy(fields.url, argv[2]);
      strcpy(fields.type, argv[3]);
      break;
    }

  /* Create the XML string */
  create_xml(&fields, &options);

  /* Send the data */
  ret = send_httppost(&fields, &options);

  return(ret);
}
